@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,7 +20,7 @@ import sys
 
 try:
     uid = os.geteuid()
-except:
+except Exception:
     pass
 else:
     if uid == 0:
@@ -33,7 +32,7 @@ else:
 if os.getenv("EXAILE_NO_OPTIMIZE"):
     try:
         sys.dont_write_bytecode = True
-    except:
+    except AttributeError:
         pass
 
 if sys.platform == 'linux2':
@@ -42,9 +41,16 @@ if sys.platform == 'linux2':
         import ctypes
         libc = ctypes.CDLL('libc.so.6')
         libc.prctl(15, 'exaile', 0, 0, 0) # 15 = PR_SET_NAME
-    except:
+    except Exception:
         pass
 
+# Set visible process name. Requires module "setproctitle"
+try:
+    from setproctitle import setproctitle
+    setproctitle('exaile')
+except ImportError:
+    pass
+
 # Find out the location of exaile's working directory, and insert it to sys.path
 basedir = os.path.dirname(os.path.realpath(__file__))
 if not os.path.exists(os.path.join(basedir, "exaile.py")):
