@@ -32,11 +32,20 @@ import menu
 import os
 import urllib
 import hashlib
-from xl import event, settings, xdg, providers
-from xl import trax as xltrack
+from xl import (
+    event,
+    settings,
+    providers,
+    trax as xltrack,
+    xdg
+)
 from xl.covers import CoverSearchMethod
 from xl.nls import gettext as _
-from xlgui import guiutil, panel
+from xlgui import (
+    icons,
+    panel
+)
+from xlgui.widgets.common import DragTreeView
 
 JAMENDO_NOTEBOOK_PAGE = None
 COVERS_METHOD = None
@@ -138,9 +147,10 @@ class JamendoPanel(panel.Panel):
 
         #setup images
         window = gtk.Window()
-        self.artist_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path("images/artist.png"))
+        self.artist_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path("images/16x16/artist.png"))
         self.album_image = window.render_icon(gtk.STOCK_CDROM, gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.title_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path('images/track.png'))
+        self.title_image = icons.MANAGER.pixbuf_from_icon_name(
+            'audio-x-generic', gtk.ICON_SIZE_SMALL_TOOLBAR)
 
         #setup search combobox
         self.search_combobox = self.builder.get_object('searchComboBox')
@@ -165,7 +175,7 @@ class JamendoPanel(panel.Panel):
         self.set_status(self.STATUS_READY)
 
         #setup results treeview
-        self.treeview = guiutil.DragTreeView(self)
+        self.treeview = DragTreeView(self)
         self.treeview.connect("row-expanded", self.row_expanded)
         self.treeview.set_headers_visible(False)
         container = self.builder.get_object('treeview_box')
@@ -383,7 +393,7 @@ class JamendoPanel(panel.Panel):
             tr.set_tag_raw('artist', track.artist_name)
             tr.set_tag_raw('album', track.album_name)
             xltrack_list.append(tr)
-        self.exaile.gui.main.get_selected_playlist().playlist.add_tracks(xltrack_list)
+        self.exaile.gui.main.get_selected_page().playlist.extend(xltrack_list)
 
     #dragdrop stuff
     def drag_data_received(self, *e):
