@@ -66,7 +66,7 @@ def get_collection_by_loc(loc):
 
         :param loc: Location of the collection
         :return: collection at location or None
-        :rtype: Collection
+        :rtype: :class:`Collection`
     """
     for c in COLLECTIONS:
         if c.loc_is_member(loc):
@@ -94,15 +94,6 @@ class CollectionScanThread(common.ProgressThread):
         self.collection.stop_scan()
         common.ProgressThread.stop(self)
 
-    def on_scan_progress_update(self, type, collection, progress):
-        """
-            Notifies about progress changes
-        """
-        if progress < 100:
-            self.emit('progress-update', progress)
-        else:
-            self.emit('done')
-
     def run(self):
         """
             Runs the thread
@@ -112,11 +103,18 @@ class CollectionScanThread(common.ProgressThread):
 
         self.collection.rescan_libraries()
 
-        # idle_add is a hack to ensure it only gets removed AFTER all
-        # events have been sent. remove when we get a better solution.
-        glib.idle_add(event.remove_callback, self.on_scan_progress_update,
+        event.remove_callback(self.on_scan_progress_update,
             'scan_progress_update')
 
+    def on_scan_progress_update(self, type, collection, progress):
+        """
+            Notifies about progress changes
+        """
+        if progress < 100:
+            self.emit('progress-update', progress)
+        else:
+            self.emit('done')
+
 class Collection(trax.TrackDB):
     """
         Manages a persistent track database.
@@ -387,6 +385,7 @@ class LibraryMonitor(gobject.GObject):
         self.__root = gio.File(library.location)
         self.__monitored = False
         self.__monitors = {}
+        self.__queue = []
         self.__lock = threading.RLock()
 
     def do_get_property(self, property):
@@ -407,7 +406,7 @@ class LibraryMonitor(gobject.GObject):
                 self.__monitored = value
                 update_thread = threading.Thread(target=self.__update_monitors)
                 update_thread.daemon = True
-                update_thread.start()
+                glib.idle_add(update_thread.start)
         else:
             raise AttributeError('unkown property %s' % property.name)
 
@@ -439,9 +438,15 @@ class LibraryMonitor(gobject.GObject):
         """
             Updates the library on changes of the location
         """
-        if event == gio.FILE_MONITOR_EVENT_CREATED:
-            added_tracks = trax.util.get_tracks_from_uri(gfile.get_uri())
-            self.__library.collection.add_tracks(added_tracks)
+        if event == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+            if gfile in self.__queue:
+                added_tracks = trax.util.get_tracks_from_uri(gfile.get_uri())
+                self.__library.collection.add_tracks(added_tracks)
+                self.__queue.remove(gfile)
+        elif event == gio.FILE_MONITOR_EVENT_CREATED:
+            # Enqueue tracks retrieval
+            if gfile not in self.__queue:
+                self.__queue += [gfile]
 
             # Set up new monitor if directory
             fileinfo = gfile.query_info('standard::type')
@@ -465,7 +470,7 @@ class LibraryMonitor(gobject.GObject):
                 removed_tracks += [track]
             else:
                 # Deleted file was most likely a directory
-                for track in self.collection:
+                for track in self.__library.collection:
                     track_gfile = gio.File(track.get_loc_for_io())
 
                     if track_gfile.has_prefix(gfile):
@@ -508,8 +513,6 @@ class Library(object):
 
             :param location: the directory this library will scan
             :type location: string
-            :param collection: the collection to associate with
-            :type collection: :class:`Collection`
             :param monitored: whether the library should update its
                 collection at changes within the library's path
             :type monitored: bool
@@ -562,6 +565,8 @@ class Library(object):
             :type monitored: bool
         """
         self.monitor.props.monitored = monitored
+        self.collection.serialize_libraries()
+        self.collection._dirty = True
 
     monitored = property(get_monitored, set_monitored)
 
@@ -623,20 +628,20 @@ class Library(object):
             return
 
         def joiner(value):
-            if not value or type(value) in (str, unicode):
+            if not value or isinstance(value, basestring):
                 return value
             else:
                 try:
                     return u"\u0000".join(value)
-                except TypeError:
-                    return value
+                except UnicodeDecodeError:
+                    return "\0".join(value)
 
         try:
             basedir = joiner(tr.get_tag_raw('__basedir'))
             album = joiner(tr.get_tag_raw('album'))
             artist = joiner(tr.get_tag_raw('artist'))
-        except UnicodeDecodeError: #TODO: figure out why this happens
-            logger.warning("Encoding error, skipping compilation check")
+        except Exception:
+            logger.warning("Error while checking for compilation: " + `tr`)
             return
         if not basedir or not album or not artist: return
         album = album.lower()
@@ -651,12 +656,12 @@ class Library(object):
             common.log_exception(log=logger)
             return
 
-        if ccheck[basedir][album] and not \
-            artist in ccheck[basedir][album]:
+        if ccheck[basedir][album] and \
+                artist not in ccheck[basedir][album]:
             if not (basedir, album) in compilations:
                 compilations.append((basedir, album))
-                logger.info("Compilation %(album)s detected in %(dir)s" %
-                        {'album':album, 'dir':basedir})
+                logger.info("Compilation %(album)r detected in %(dir)r" %
+                        {'album': album, 'dir': basedir})
 
         ccheck[basedir][album].append(artist)
 
@@ -664,7 +669,8 @@ class Library(object):
         """
             Rescan the track at a given location
 
-            :gloc: A gio.File representing the location
+            :param gloc: the location
+            :type gloc: :class:`gio.File`
 
             returns: the Track object, None if it could not be updated
         """
@@ -758,11 +764,11 @@ class Library(object):
 
             # progress update
             if notify_interval is not None and count % notify_interval == 0:
-                event.log_event_sync('tracks_scanned', self, count)
+                event.log_event('tracks_scanned', self, count)
 
         # final progress update
         if notify_interval is not None:
-            event.log_event_sync('tracks_scanned', self, count)
+            event.log_event('tracks_scanned', self, count)
 
 
 
