@@ -24,18 +24,30 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+Provides the base for obtaining and storing covers, also known 
+as album art.
+"""
+
+import glib
+import gio
 import logging
-import os
 import hashlib
+import os
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
 
-import glib
-import gio
-
-from xl import common, providers, event, settings, xdg, trax
+from xl.nls import gettext as _
+from xl import (
+    common,
+    event,
+    providers,
+    settings,
+    trax,
+    xdg
+)
 
 logger = logging.getLogger(__name__)
 
@@ -121,7 +133,6 @@ class CoverManager(providers.ProviderHandler):
         self.load()
         for method in self.get_providers():
             self.on_provider_added(method)
-        self.__save_timer_id = 0
 
         default_cover_file = open(xdg.get_data_path('images', 'nocover.png'), 'rb')
         self.default_cover_data = default_cover_file.read()
@@ -200,6 +211,19 @@ class CoverManager(providers.ProviderHandler):
             return None
         return (tag, tuple(value))
 
+    def get_db_string(self, track):
+        """
+            Returns the internal string used to map the cover
+            to a track
+
+            :param track: the track to retrieve the string for
+            :type track: :class:`xl.trax.Track`
+            :returns: the internal identifier string
+        """
+        key = self._get_track_key(track)
+
+        return self.db.get(key)
+
     @common.synchronized
     @common.cached(5)
     def find_covers(self, track, limit=-1, local_only=False):
@@ -242,7 +266,7 @@ class CoverManager(providers.ProviderHandler):
         key = self._get_track_key(track)
         if key:
             self.db[key] = db_string
-            self.__set_save_timeout()
+            self.timeout_save()
             event.log_event('cover_set', self, track)
 
     def remove_cover(self, track):
@@ -256,7 +280,7 @@ class CoverManager(providers.ProviderHandler):
         if db_string:
             del self.db[key]
             self.__cache.remove(db_string)
-            self.__set_save_timeout()
+            self.timeout_save()
             event.log_event('cover_removed', self, track)
 
     def get_cover(self, track, save_cover=True, set_only=False,
@@ -277,8 +301,7 @@ class CoverManager(providers.ProviderHandler):
         if track is None:
             return self.get_default_cover() if use_default else None
 
-        key = self._get_track_key(track)
-        db_string = self.db.get(key)
+        db_string = self.get_db_string(track)
         if db_string:
             return self.get_cover_data(db_string, use_default=use_default)
 
@@ -346,13 +369,14 @@ class CoverManager(providers.ProviderHandler):
         if data:
             self.db = data
 
+    @common.glib_wait_seconds(60)
+    def timeout_save(self):
+        self.save()
+
     def save(self):
         """
             Save the db
         """
-        if self.__save_timer_id:
-            glib.source_remove(self.__save_timer_id)
-
         path = os.path.join(self.location, 'covers.db')
         try:
             f = open(path + ".new", 'wb')
@@ -370,11 +394,6 @@ class CoverManager(providers.ProviderHandler):
         except OSError:
             pass
 
-    def __set_save_timeout(self):
-        if self.__save_timer_id:
-            glib.source_remove(self.__save_timer_id)
-        self.__save_timer_id = glib.timeout_add_seconds(60, self.save)
-
     def on_provider_added(self, provider):
         self.methods[provider.name] = provider
         if provider.name not in self.order:
@@ -444,44 +463,37 @@ class TagCoverFetcher(CoverSearchMethod):
     """
     use_cache = False
     name = "tags"
+    title = _('Tags')
     cover_tags = ["cover", "coverart"]
     fixed = True
     fixed_priority = 30
+
     def find_covers(self, track, limit=-1):
-        data = None
+        covers = [] 
         tagname = None
+        uri = track.get_loc_for_io()
+
         for tag in self.cover_tags:
             try:
-                data = track.get_tag_disk(tag)
+                # Force type conversion to list, fails for None
+                covers = list(track.get_tag_disk(tag))
                 tagname = tag
                 break
-            except KeyError:
+            except (TypeError, KeyError):
                 pass
-        if data:
-            # path format: tagname:track_uri
-            path = "%s:%s"%(tagname, track.get_loc_for_io())
-            return [path]
-        return []
+
+        return ['{tagname}:{index}:{uri}'.format(tagname=tagname, index=index, uri=uri) \
+            for index in range(0, len(covers))]
 
     def get_cover_data(self, db_string):
-        tag, uri = db_string.split(":", 1)
-        tr = trax.Track(uri, scan=False)
-        data = tr.get_tag_disk(tag)
-        if not data:
+        tag, index, uri = db_string.split(':', 2)
+        track = trax.Track(uri, scan=False)
+        covers = track.get_tag_disk(tag)
+
+        if not covers:
             return None
-        if isinstance(data, list):
-            data = data[0]
-        b64_valid = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" + \
-                "01234567890+/"
-        for c in data.rstrip("="):
-            if c not in b64_valid:
-                break
-        else:
-            try:
-                data = data.decode("base64")
-            except UnicodeDecodeError:
-                pass
-        return data
+
+        return covers[int(index)].data
 
 class LocalFileCoverFetcher(CoverSearchMethod):
     """
@@ -490,11 +502,19 @@ class LocalFileCoverFetcher(CoverSearchMethod):
     """
     use_cache = False
     name = "localfile"
+    title = _('Local file')
     uri_types = ['file', 'smb', 'sftp', 'nfs']
     extensions = ['.png', '.jpg', '.jpeg', '.gif']
-    preferred_names = ['album', 'cover']
+    preferred_names = []
     fixed = True
     fixed_priority = 31
+
+    def __init__(self):
+        CoverSearchMethod.__init__(self)
+
+        event.add_callback(self.on_option_set, 'covers_localfile_option_set')
+        self.on_option_set('covers_localfile_option_set', settings, 'covers/localfile/preferred_names')
+
     def find_covers(self, track, limit=-1):
         # TODO: perhaps should instead check to see if its mounted in
         # gio, rather than basing this on uri type. file:// should
@@ -502,8 +522,11 @@ class LocalFileCoverFetcher(CoverSearchMethod):
         if track.get_type() not in self.uri_types:
             return []
         basedir = gio.File(track.get_loc_for_io()).get_parent()
-        if not basedir.query_info("standard::type").get_file_type() == \
-                gio.FILE_TYPE_DIRECTORY:
+        try:
+            if not basedir.query_info("standard::type").get_file_type() == \
+                    gio.FILE_TYPE_DIRECTORY:
+                return []
+        except gio.Error:
             return []
         covers = []
         for fileinfo in basedir.enumerate_children("standard::type"
@@ -532,8 +555,16 @@ class LocalFileCoverFetcher(CoverSearchMethod):
         except glib.GError:
             return None
 
+    def on_option_set(self, e, settings, option):
+        """
+            Updates the internal settings upon option change
+        """
+        if option == 'covers/localfile/preferred_names':
+            self.preferred_names = settings.get_option(option, ['album', 'cover'])
 
 
+
+#: The singleton :class:`CoverManager` instance
 MANAGER = CoverManager(location=xdg.get_data_home_path("covers",
         check_exists=False))
 
