@@ -31,12 +31,7 @@ Provides a signals-like system for sending and listening for 'events'
 Events are kind of like signals, except they may be listened for on a
 global scale, rather than connected on a per-object basis like signals
 are. This means that ANY object can emit ANY event, and these events may
-be listened for by ANY object. Events may be emitted either syncronously
-or asyncronously, the default is asyncronous.
-
-The events module also provides an idle_add() function similar to that of
-glib. However this should not be used for long-running tasks as they
-may block other events queued via idle_add().
+be listened for by ANY object.
 
 Events should be emitted AFTER the given event has taken place. Often the
 most appropriate spot is immediately before a return statement.
@@ -67,43 +62,42 @@ class Nothing(object):
 
 _NONE = Nothing() # used by event for a safe None replacement
 
-def log_event(type, obj, data, async=True):
+def log_event(type, obj, data):
     """
         Sends an event.
 
-        type: the 'type' or 'name' of the event. [string]
-        obj: the object sending the event. [object]
-        data: some data about the event, None if not required [object]
-        async: whether or not to emit asyncronously. [bool]
+        :param type: the *type* or *name* of the event.
+        :type type: string
+        :param obj: the object sending the event.
+        :type obj: object
+        :param data: some data about the event, None if not required
+        :type data: object
     """
     global EVENT_MANAGER
     e = Event(type, obj, data, time.time())
-    if async:
-        EVENT_MANAGER.emit_async(e)
-    else:
-        EVENT_MANAGER.emit(e)
-
-def log_event_sync(type, obj, data, async=False):
-    # same as log_event, but defaults to synchronous
-    log_event(type, obj, data, async)
+    EVENT_MANAGER.emit(e)
 
 def add_callback(function, type=None, obj=None, *args, **kwargs):
     """
-        Sets an Event callback
+        Adds a callback to an event
 
         You should ALWAYS specify one of the two options on what to listen
         for. While not forbidden to listen to all events, doing so will
         cause your callback to be called very frequently, and possibly may
         cause slowness within the player itself.
 
-        @param function: the function to call when the event happens [function]
-        @param type: the 'type' or 'name' of the event to listen for, eg
-                "tracks_added",  "cover_changed". Defaults to any event if
-                not specified. [string]
-        @param obj: the object to listen to events from, eg exaile.collection,
-                exaile.cover_manager. Defaults to any object if not
-                specified. [object]
-        Any additional paramaters will be passed to the callback.
+        :param function: the function to call when the event happens
+        :type function: callable
+        :param type: the *type* or *name* of the event to listen for, eg
+                `tracks_added`, `cover_changed`. Defaults to any event if
+                not specified.
+        :type type: string
+        :param obj: the object to listen to events from, e.g. `exaile.collection`
+                or `xl.covers.MANAGER`. Defaults to any object if not
+                specified.
+        :type obj: object
+
+        Any additional parameters will be passed to the callback.
     """
     global EVENT_MANAGER
     EVENT_MANAGER.add_callback(function, type, obj, args, kwargs)
@@ -254,9 +248,7 @@ class EventManager(object):
                     if not cb.valid:
                         try:
                             self.callbacks[event.type][event.object].remove(cb)
-                        except KeyError:
-                            pass
-                        except ValueError:
+                        except (KeyError, ValueError):
                             pass
                     elif event.time >= cb.time:
                         if self.use_logger and (not self.logger_filter or \
@@ -268,7 +260,7 @@ class EventManager(object):
                                         'event': event.type})
                         cb.wfunction().__call__(event.type, event.object,
                                 event.data, *cb.args, **cb.kwargs)
-                except:
+                except Exception:
                     # something went wrong inside the function we're calling
                     common.log_exception(logger,
                                 message="Event callback exception caught!")
