@@ -25,68 +25,62 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+Provides an extensible framework for processing and
+preparation of data for display in various contexts.
+"""
+
+from datetime import date
 import gio
 import glib
 import gobject
 import re
-from datetime import date
 from string import Template, _TemplateMetaclass
 
-from xl import event, main, providers, settings, trax
+from xl import (
+    common,
+    event,
+    main,
+    providers,
+    settings,
+    trax
+)
 from xl.common import TimeSpan
 from xl.nls import gettext as _, ngettext
 
 class _ParameterTemplateMetaclass(_TemplateMetaclass):
+    # Allows for $tag, ${tag}, ${tag:parameter} and ${tag:parameter=argument}
     pattern = r"""
     %(delim)s(?:
-      (?P<escaped>%(delim)s) |   # Escape sequence of two delimiters
-      (?P<named>%(id)s)      |   # delimiter and a Python identifier
-      {(?P<braced>%(id_param)s)}   |   # delimiter and a braced identifier
-      (?P<invalid>)              # Other ill-formed delimiter exprs
-    )
-    """
-    # Allows for $tag, ${tag}, ${tag:parameter} and ${tag:parameter=argument}
-    match_pattern = r"""
-    %(delim)s(?:
-      (?P<escaped>%(delim)s) |   # Escape sequence of two delimiters
-      (?P<named>%(id)s)      |   # Delimiter and a Python identifier
+      (?P<escaped>%(delim)s) |     # Escape sequence of two delimiters
+      (?P<named>%(id)s)      |     # Delimiter and a Python identifier
       {
-        (?P<braced>%(id)s)       # Delimiter and a braced identifier
+        (?P<braced>%(id)s)         # Delimiter and a braced identifier
         (!?:
           (?P<parameters>
-            %(id)s               # First optional parameter indicated with ':'
-            (!?=[^,}=]*)?        # Optional argument indicated with '='
-            (!?,\s*%(id)s        # Further optional parameters separated with ','
-              (!?=[^,}=]*)?      # Optional argument indicated with '='
+            %(id)s                 # First optional parameter indicated with ':'
+            (!?=(!?%(arg)s)+?)?    # Optional argument indicated with '='
+            (!?,\s*%(id)s          # Further optional parameters separated with ','
+              (!?=(!?%(arg)s)+?)?  # Optional argument indicated with '='
             )*      
           )
         )?
       }                      |
-      (?P<invalid>)              # Other ill-formed delimiter expressions
+      (?P<invalid>)                # Other ill-formed delimiter expressions
     )
     """
-
     def __init__(cls, name, bases, dct):
         super(_ParameterTemplateMetaclass, cls).__init__(name, bases, dct)
         if 'pattern' in dct:
             pattern = cls.pattern
         else:
             pattern = _ParameterTemplateMetaclass.pattern % {
-                'delim'   : re.escape(cls.delimiter),
-                'id'      : cls.idpattern,
-                'id_param': cls.idpattern_param
+                'delim': re.escape(cls.delimiter),
+                'id': cls.idpattern,
+                'arg': cls.argpattern,
             }
         cls.pattern = re.compile(pattern, re.IGNORECASE | re.VERBOSE)
 
-        if 'match_pattern' in dct:
-            match_pattern = cls.match_pattern
-        else:
-            match_pattern = _ParameterTemplateMetaclass.match_pattern % {
-                'delim'   : re.escape(cls.delimiter),
-                'id'      : cls.idpattern
-            }
-        cls.match_pattern = re.compile(match_pattern, re.IGNORECASE | re.VERBOSE)
-
 class ParameterTemplate(Template):
     """
         An extended template class which additionally
@@ -94,34 +88,88 @@ class ParameterTemplate(Template):
 
         This introduces another pattern group named
         "parameters" in addition to the groups
-        created by string.Template.
+        created by :class:`string.Template`
 
         Examples:
-        * ${foo:parameter1}
-        * ${bar:parameter1, parameter2}
-        * ${qux:parameter1=argument1, parameter2}
+
+        * ``${foo:parameter1}``
+        * ``${bar:parameter1, parameter2}``
+        * ``${qux:parameter1=argument1, parameter2}``
     """
     __metaclass__ = _ParameterTemplateMetaclass
-
-    idpattern_param = r'[_a-z][_a-z0-9:=,]*'
+    argpattern = r'[^,}=]|\,|\}|\='
 
     def __init__(self, template):
         """
-            :param template: The template string
+            :param template: the template string
         """
         Template.__init__(self, template)
 
+    def safe_substitute(self, *args, **kwargs):
+        """
+            Overridden to allow for parametrized identifiers
+        """
+        if len(args) > 1:
+            raise TypeError('Too many positional arguments')
+
+        if not args:
+            mapping = kwargs
+        elif kwargs:
+            mapping = _multimap(kwargs, args[0])
+        else:
+            mapping = args[0]
+
+        # Helper function for .sub()
+        def convert(mo):
+            named = mo.group('named')
+
+            if named is not None:
+                try:
+                    # We use this idiom instead of str() because the latter
+                    # will fail if val is a Unicode containing non-ASCII
+                    return '%s' % (mapping[named],)
+                except KeyError:
+                    return self.delimiter + named
+
+            braced = mo.group('braced')
+
+            if braced is not None:
+                parts = [braced]
+                parameters = mo.group('parameters')
+
+                if parameters is not None:
+                    parts += [parameters]
+
+                try:
+                    return '%s' % (mapping[':'.join(parts)],)
+                except KeyError:
+                    return self.delimiter + '{' + ':'.join(parts) + '}'
+
+            if mo.group('escaped') is not None:
+                return self.delimiter
+
+            if mo.group('invalid') is not None:
+                return self.delimiter
+
+            raise ValueError('Unrecognized named group in pattern',
+                             self.pattern)
+
+        return self.pattern.sub(convert, self.template)
+
 class Formatter(gobject.GObject):
     """
         A generic text formatter based on a format string
+
+        By default the following parameters are provided
+        to each identifier:
+
+        * ``prefix``, ``suffix``: a string to put before or after the formatted string if that string is not empty
+            * Whitespace will be not be touched and transferred as is
+            * The characters ``,``, ``}`` and ``=`` need to be escaped like ``\,``, ``\}`` and ``\=`` respectively
+        * ``pad``: desired length the formatted string should have, will be achieved using the ``padstring``
+        * ``padstring``: a string to use for padding, will be repeated as often as possible to achieve the desired length specified by ``pad``
+            * Example: ``${identifier:pad=4, padstring=XY}`` for *identifier* having the value *a* will become *XYXa*
     """
-    __gsignals__ = {
-        'format-changed': (
-            gobject.SIGNAL_RUN_LAST,
-            gobject.TYPE_NONE,
-            (gobject.TYPE_STRING,)
-        )
-    }
     __gproperties__ = {
         'format': (
             gobject.TYPE_STRING,
@@ -133,13 +181,10 @@ class Formatter(gobject.GObject):
     }
     def __init__(self, format):
         """
-            :param format: The initial format, see the documentation
-                of string.Template for details
+            :param format: the initial format, see the documentation
+                of :class:`string.Template` for details
             :type format: string
         """
-        if self.__class__.__name__ == 'Formatter':
-            raise TypeError("cannot create instance of abstract "
-                            "(non-instantiable) type `Formatter'")
         gobject.GObject.__init__(self)
 
         self._template = ParameterTemplate(format)
@@ -161,173 +206,217 @@ class Formatter(gobject.GObject):
         if property.name == 'format':
             if value != self._template.template:
                 self._template.template = value
-                self.emit('format-changed', value)
         else:
             raise AttributeError('unkown property %s' % property.name)
 
-    def substitute(self, text, replacement):
+    def extract(self):
         """
-            Returns the replacement of a text
+            Retrieves the identifiers and their optional parameters
 
-            :param text: The text to replace
-            :type text: string
-            :param replacement: The replacement
-            :type replacement: string or callable
-            :returns: The replacement
-            :rtype: string
+            Example of the returned dictionary::
+
+                extractions = {
+                    'identifier1': (
+                        'identifier1', {}),
+                    'identifier2:parameter': (
+                        'identifier2', {'parameter': True}),
+                    'identifier3:parameter=argument': (
+                        'identifier3', {'parameter': 'argument'})
+                }
+
+            :returns: the extractions
+            :rtype: dict
         """
-        if callable(replacement):
-            return replacement(text)
+        matches = self._template.pattern.finditer(self._template.template)
+        extractions = {}
 
-        return replacement
+        # Extract list of identifiers and parameters from the format string
+        for match in matches:
+            groups = match.groupdict()
+
+            # We only care about braced and named, not escaped and invalid
+            identifier = groups['braced'] or groups['named']
+
+            if identifier is None:
+                continue
+
+            identifier_parts = [identifier]
+            parameters = {}
+
+            if groups['parameters'] is not None:
+                # Split parameters on unescaped comma
+                parameters = [p.lstrip() \
+                    for p in re.split(r'(?<!\\),', groups['parameters'])]
+                # Split arguments on unescaped equals sign
+                parameters = [(re.split(r'(?<!\\)=', p, 1) + [True])[:2] \
+                    for p in parameters]
+                # Turn list of lists into a proper dictionary
+                parameters = dict(parameters)
+
+                # Remove now obsolete escapes
+                for p in parameters:
+                    argument = parameters[p]
+
+                    if type(argument) is not bool:
+                        argument = argument.replace(r'\,', ',')
+                        argument = argument.replace(r'\}', '}')
+                        argument = argument.replace(r'\=', '=')
+                        parameters[p] = argument
+
+                identifier_parts += [groups['parameters']]
+
+            # Required to make multiple occurences of the same
+            # identifier with different parameters work
+            extractions[':'.join(identifier_parts)] = (identifier, parameters)
+
+        return extractions
 
     def format(self, *args):
         """
             Returns a string by formatting the passed data
 
-            :param args: Data to base the formatting on
-            :returns: The formatted text
+            :param args: data to base the formatting on
+            :returns: the formatted text
             :rtype: string
         """
-        pass
+        extractions = self.extract()
+        substitutions = {}
+
+        for needle, (identifier, parameters) in extractions.iteritems():
+            substitute = None
+
+            if needle in self._substitutions:
+                substitute = self._substitutions[needle]
+            elif identifier in self._substitutions:
+                substitute = self._substitutions[identifier]
+
+            if substitute is not None:
+                prefix = parameters.pop('prefix', '')
+                suffix = parameters.pop('suffix', '')
+                pad = int(parameters.pop('pad', 0))
+                padstring = parameters.pop('padstring', '')
+
+                if callable(substitute):
+                    substitute = substitute(*args, **parameters)
+
+                if pad > 0 and padstring:
+                    # Decrease pad length by value length
+                    pad = max(0, pad - len(substitute))
+                    # Retrieve the maximum multiplier for the pad string
+                    padcount = pad / len(padstring) + 1
+                    # Generate pad string
+                    padstring = padcount * padstring
+                    # Clamp pad string
+                    padstring = padstring[0:pad]
+                    substitute = '%s%s' % (padstring, substitute)
+
+                if substitute:
+                    substitute = '%s%s%s' % (prefix, substitute, suffix)
+
+                substitutions[needle] = substitute
+
+        return self._template.safe_substitute(substitutions)
 
 class ProgressTextFormatter(Formatter):
     """
         A text formatter for progress indicators
     """
-    def __init__(self, format):
-        """
-            :param format: The initial format, see the documentation
-                of string.Template for details
-            :type format: string
-        """
+    def __init__(self, format, player):
         Formatter.__init__(self, format)
-
-        try:
-            exaile = main.exaile()
-        except AttributeError:
-            event.add_callback(self.on_exaile_loaded, 'exaile_loaded')
-        else:
-            self.on_exaile_loaded('exaile_loaded', exaile, None)
+        self._player = player
 
     def format(self, current_time=None, total_time=None):
         """
             Returns a string suitable for progress indicators
 
-            :param current_time: the current progress
+            :param current_time: the current progress, taken from the current playback if not set
             :type current_time: float
-            :param total_time: the total length of a track
+            :param total_time: the total length of a track, taken from the current playback if not set
             :type total_time: float
             :returns: The formatted text
             :rtype: string
         """
+        total_remaining_time = 0
+        
         if current_time is None:
-            current_time = self.player.get_time()
+            current_time = self._player.get_time()
 
         if total_time is None:
-            total_time = self.player.current.get_tag_raw('__length')
+            track = self._player.current
+
+            if track is not None:
+                total_time = track.get_tag_raw('__length')
 
         if total_time is None:
-            if not self.player.current.is_local():
-                return _('Streaming...')
-
             total_time = remaining_time = 0
         else:
             remaining_time = total_time - current_time
 
-        self._substitutions = {
-            'current_time': LengthTagFormatter.format_value(current_time),
-            'remaining_time': LengthTagFormatter.format_value(remaining_time),
-            'total_time': LengthTagFormatter.format_value(total_time)
-        }
-        
-        return self._template.safe_substitute(self._substitutions)
+        playlist = self._player.queue.current_playlist
 
-    def on_exaile_loaded(self, e, exaile, nothing):
-        """
-            Sets up references after controller is loaded
-        """
-        self.player = exaile.player
-        event.remove_callback(self.on_exaile_loaded, 'exaile_loaded')
+        if playlist and playlist.current_position >= 0:        
+            tracks = playlist[playlist.current_position:]
+            total_remaining_time = sum([t.get_tag_raw('__length') \
+                for t in tracks if t.get_tag_raw('__length')])
+            total_remaining_time -= current_time
 
-class TrackFormatter(Formatter, providers.ProviderHandler):
+        self._substitutions['current_time'] = \
+            LengthTagFormatter.format_value(current_time)
+        self._substitutions['remaining_time'] = \
+            LengthTagFormatter.format_value(remaining_time)
+        self._substitutions['total_time'] = \
+            LengthTagFormatter.format_value(total_time)
+        self._substitutions['total_remaining_time'] = \
+            LengthTagFormatter.format_value(total_remaining_time)
+
+        return Formatter.format(self)
+
+class TrackFormatter(Formatter):
     """
         A formatter for track data
     """
-    def __init__(self, format):
-        """
-            :param format: The initial format, see the documentation
-                of string.Template for details
-            :type format: string
-        """
-        Formatter.__init__(self, format)
-        providers.ProviderHandler.__init__(self, 'tag_formatting')
-
     def format(self, track, markup_escape=False):
         """
-            Returns a string suitable for progress indicators
+            Returns a string for places where
+            track data is presented to the user
 
-            :param track: A single track to take data from
+            :param track: a single track to take data from
             :type track: :class:`xl.trax.Track`
-            :param markup_escape: Whether to escape markup-like
+            :param markup_escape: whether to escape markup-like
                 characters in tag values
             :type markup_escape: bool
-            :returns: The formatted text
+            :returns: the formatted text
             :rtype: string
         """
         if not isinstance(track, trax.Track):
             raise TypeError('First argument to format() needs '
                             'to be of type xl.trax.Track')
 
-        matches = self._template.match_pattern.finditer(self._template.template)
-        tags = {}
+        extractions = self.extract()
         self._substitutions = {}
 
-        # Extract list of tags contained in the format string
-        for match in matches:
-            groups = match.groupdict()
-
-            # We don't care about escaped and invalid
-            if groups['braced'] is not None:
-                tag = groups['braced']
-            elif groups['named'] is not None:
-                tag = groups['named']
-            else:
-                continue
-
-            idparts = [tag]
-            parameters = {}
-
-            if groups['parameters'] is not None:
-                parameters = groups['parameters'].split(',')
-                # Turns [['foo', 'arg'], ['bar']] into {'foo': 'arg', 'bar': True}
-                parameters = dict([(p.split('=', 1) + [True])[:2] for p in parameters])
-                idparts += [groups['parameters']]
-
-            tags[':'.join(idparts)] = (tag, parameters)
-
-        for id, (tag, parameters) in tags.iteritems():
-            provider = self.get_provider(tag)
+        for identifier, (tag, parameters) in extractions.iteritems():
+            provider = providers.get_provider('tag-formatting', tag)
 
             if provider is None:
-                self._substitutions[id] = track.get_tag_display(tag)
+                substitute = track.get_tag_display(tag)
             else:
-                self._substitutions[id] = provider.format(track, parameters)
+                substitute = provider.format(track, parameters)
 
-        if markup_escape:
-            for id in self._substitutions.iterkeys():
-                self._substitutions[id] = glib.markup_escape_text(
-                    self._substitutions[id])
+            if markup_escape:
+                substitute = glib.markup_escape_text(substitute)
 
-        return self._template.safe_substitute(self._substitutions)
+            self._substitutions[identifier] = substitute
+
+        return Formatter.format(self)
 
 class TagFormatter():
     """
-        A formatter for a tag of a track
+        A formatter provider for a tag of a track
     """
     def __init__(self, name):
         """
-            :param name: The name of the tag
+            :param name: the name of the tag
             :type name: string
         """
         self.name = name
@@ -342,9 +431,6 @@ class TagFormatter():
             :type track: :class:`xl.trax.Track`
             :param parameters: optionally passed parameters
             :type parameters: dictionary
-                Parameters specified via 'parameter=value'
-                will be directly mapped into the dictionary,
-                parameters without argument will be set to True
             :returns: the formatted value
             :rtype: string
         """
@@ -364,9 +450,6 @@ class TrackNumberTagFormatter(TagFormatter):
             :param track: the track to get the tag from
             :type track: :class:`xl.trax.Track`
             :param parameters: optionally passed parameters
-                Possible values are:
-                * pad: n [n being an arbitrary number]
-                  Influences the amount of leading zeros
             :type parameters: dictionary
             :returns: the formatted value
             :rtype: string
@@ -376,41 +459,21 @@ class TrackNumberTagFormatter(TagFormatter):
         if not value:
             return ''
 
-        pad = parameters.get('pad', 1)
-
-        return self.format_value(value, pad)
-
-    @staticmethod
-    def format_value(value, pad=1):
-        """
-            Formats a tracknumber value
-
-            :param value: A tracknumber
-            :type value: int or string
-            :param pad: Amount of leading zeros
-            :type pad: int
-            :returns: the formatted value
-            :rtype: string
-        """
-        try:
-            pad = int(pad)
-        except ValueError: # No int
-            pad = 1
-
         try: # n/n
             value, count = value.split('/')
         except ValueError: # n
             pass
 
-        format_string = '%%0%(pad)dd' % {'pad': pad}
+        if not value:
+            return ''
 
         try:
-            value = format_string % int(value)
-        except ValueError: # Invalid number
-            pass
+            value = int(value)
+        except ValueError:
+            return ''
 
-        return value
-providers.register('tag_formatting', TrackNumberTagFormatter())
+        return '%d' % value
+providers.register('tag-formatting', TrackNumberTagFormatter())
 
 class DiscNumberTagFormatter(TagFormatter):
     """
@@ -426,50 +489,30 @@ class DiscNumberTagFormatter(TagFormatter):
             :param track: the track to get the tag from
             :type track: :class:`xl.trax.Track`
             :param parameters: optionally passed parameters
-                Possible values are:
-                * pad: n [n being an arbitrary number]
-                  Influences the amount of leading zeros
             :type parameters: dictionary
             :returns: the formatted value
             :rtype: string
         """
         value = track.get_tag_raw(self.name, join=True)
+
         if not value:
             return ''
-        pad = parameters.get('pad', 1)
-        return self.format_value(value, pad)
-
-    @staticmethod
-    def format_value(value, pad=1):
-        """
-            Formats a tracknumber value
-
-            :param value: A tracknumber
-            :type value: int or string
-            :param pad: Amount of leading zeros
-            :type pad: int
-            :returns: the formatted value
-            :rtype: string
-        """
-        try:
-            pad = int(pad)
-        except ValueError: # No int
-            pad = 1
 
         try: # n/n
             value, count = value.split('/')
         except ValueError: # n
             pass
 
-        format_string = '%%0%(pad)dd' % {'pad': pad}
+        if not value:
+            return ''
 
         try:
-            value = format_string % int(value)
-        except ValueError: # Invalid number
-            pass
+            value = int(value)
+        except ValueError:
+            return ''
 
-        return value
-providers.register('tag_formatting', DiscNumberTagFormatter())
+        return '%d' % value
+providers.register('tag-formatting', DiscNumberTagFormatter())
 
 class ArtistTagFormatter(TagFormatter):
     """
@@ -486,6 +529,7 @@ class ArtistTagFormatter(TagFormatter):
             :type track: :class:`xl.trax.Track`
             :param parameters: optionally passed parameters
                 Possible values are:
+
                 * compilate:
                   Allows for proper handling of compilations,
                   either via albumartist tag, a fallback value,
@@ -498,7 +542,7 @@ class ArtistTagFormatter(TagFormatter):
             artist_compilations=compilate)
 
         return value
-providers.register('tag_formatting', ArtistTagFormatter())
+providers.register('tag-formatting', ArtistTagFormatter())
 
 class LengthTagFormatter(TagFormatter):
     """
@@ -513,12 +557,12 @@ class LengthTagFormatter(TagFormatter):
 
             :param track: the track to get the tag from
             :type track: :class:`xl.trax.Track`
-            :param parameters: optionally passed parameters
-                Possible values are:
-                * format: (short|long|verbose)
-                  Yields "1:02:42",
-                  "1h, 2m, 42s" or
-                  "1 hour, 2 minutes, 42 seconds"
+            :param parameters: Verbosity of the output,
+                possible values for "format" are:
+
+                * short: "1:02:42"
+                * long: "1h, 2m, 42s"
+                * verbose: "1 hour, 2 minutes, 42 seconds"
             :type parameters: dictionary
             :returns: the formatted value
             :rtype: string
@@ -533,12 +577,14 @@ class LengthTagFormatter(TagFormatter):
         """
             Formats a length value
 
-            :param value: The length in seconds
+            :param value: the length in seconds
             :type value: float
-            :param format: Verbosity of the output
-                Possible values are short, long or verbose
-                yielding "1:02:42", "1h, 2m, 42s" or
-                "1 hour, 2 minutes, 42 seconds"
+            :param format: verbosity of the output,
+                possible values are:
+
+                * short: "1:02:42"
+                * long: "1h, 2m, 42s"
+                * verbose: "1 hour, 2 minutes, 42 seconds"
             :type format: string
             :returns: the formatted value
             :rtype: string
@@ -547,64 +593,50 @@ class LengthTagFormatter(TagFormatter):
         text = ''
 
         if format == 'verbose':
-            if span.years > 0:
-                text += ngettext('%d year, ', '%d years, ', span.years) % span.years
-
             if span.days > 0:
                 text += ngettext('%d day, ', '%d days, ', span.days) % span.days
-
             if span.hours > 0:
                 text += ngettext('%d hour, ', '%d hours, ', span.hours) % span.hours
-
             text += ngettext('%d minute, ', '%d minutes, ', span.minutes) % span.minutes
             text += ngettext('%d second', '%d seconds', span.seconds) % span.seconds
-        elif format == 'long':
-            if span.years > 0:
-                # TRANSLATORS: Short form of an amount of years
-                text += _('%dy, ') % span.years
 
+        elif format == 'long':
             if span.days > 0:
                 # TRANSLATORS: Short form of an amount of days
                 text += _('%dd, ') % span.days
-
             if span.hours > 0:
                 # TRANSLATORS: Short form of an amount of hours
                 text += _('%dh, ') % span.hours
-
             # TRANSLATORS: Short form of an amount of minutes
             text += _('%dm, ') % span.minutes
             # TRANSLATORS: Short form of an amount of seconds
             text += _('%ds') % span.seconds
+
         elif format == 'short':
-            durations = []
-
-            if span.years > 0:
-                durations += [span.years]
-
             if span.days > 0:
-                durations += [span.days]
+                # TRANSLATORS: Short form of an amount of days
+                text += _('%dd ') % span.days
+            if span.hours > 0 or text: # always show hours when > 1 day
+                # TRANSLATORS: Time duration (hours:minutes:seconds)
+                text += _('%d:%02d:%02d') % (
+                        span.hours, span.minutes, span.seconds)
+            else:
+                # TRANSLATORS: Time duration (minutes:seconds)
+                text += _('%d:%02d') % (span.minutes, span.seconds)
 
-            if span.hours > 0:
-                durations += [span.hours]
-
-            durations += [span.minutes, span.seconds]
-
-            first = durations.pop(0)
-            values = ['%02d' % duration for duration in durations]
-            values = ['%d' % first] + values
-
-            text = ':'.join(values)
         else:
             raise ValueError('Invalid argument "%s" passed to parameter '
                 '"format" for tag "__length", possible arguments are '
                 '"short", "long" and "verbose"' % format)
 
         return text
-providers.register('tag_formatting', LengthTagFormatter())
+providers.register('tag-formatting', LengthTagFormatter())
 
 class RatingTagFormatter(TagFormatter):
     """
         A formatter for the rating of a track
+        
+        Will return glyphs representing the rating like ★★★☆☆
     """
     def __init__(self):
         TagFormatter.__init__(self, '__rating')
@@ -627,14 +659,21 @@ class RatingTagFormatter(TagFormatter):
         empty = '☆' * int(maximum - rating)
 
         return ('%s%s' % (filled, empty)).decode('utf-8')
-providers.register('tag_formatting', RatingTagFormatter())
+providers.register('tag-formatting', RatingTagFormatter())
 
-class LastPlayedTagFormatter(TagFormatter):
+class DateTagFormatter(TagFormatter):
     """
-        A formatter for the last time a track was played
+        A generic formatter for timestamp formatting
+
+        Will return the localized string for *Today*, *Yesterday*
+        or the respective localized date for earlier dates
     """
-    def __init__(self):
-        TagFormatter.__init__(self, '__last_played')
+    def __init__(self, name):
+        """
+            :param name: the name of the tag
+            :type name: string
+        """
+        TagFormatter.__init__(self, name)
 
     def format(self, track, parameters):
         """
@@ -648,6 +687,7 @@ class LastPlayedTagFormatter(TagFormatter):
             :rtype: string
         """
         value = track.get_tag_raw(self.name)
+        # TRANSLATORS: Indicates that a track has never been played before
         text = _('Never')
 
         try:
@@ -666,14 +706,31 @@ class LastPlayedTagFormatter(TagFormatter):
                 text = last_played.strftime('%x')
 
         return text
-providers.register('tag_formatting', LastPlayedTagFormatter())
 
-class FilenameTagFormatter(TagFormatter):
+class LastPlayedTagFormatter(DateTagFormatter):
     """
-        A formatter for the filename of a track
+        A formatter for the last time a track was played
     """
     def __init__(self):
-        TagFormatter.__init__(self, 'filename')
+        DateTagFormatter.__init__(self, '__last_played')
+providers.register('tag-formatting', LastPlayedTagFormatter())
+
+class DateAddedTagFormatter(DateTagFormatter):
+    """
+        A formatter for the date a track
+        was added to the collection
+    """
+    def __init__(self):
+        DateTagFormatter.__init__(self, '__date_added')
+providers.register('tag-formatting', DateAddedTagFormatter())
+
+class LocationTagFormatter(TagFormatter):
+    """
+        A formatter for the location of a track,
+        properly sanitized if necessary
+    """
+    def __init__(self):
+        TagFormatter.__init__(self, '__loc')
 
     def format(self, track, parameters):
         """
@@ -686,9 +743,62 @@ class FilenameTagFormatter(TagFormatter):
             :returns: the formatted value
             :rtype: string
         """
-        gfile = gio.File(track.get_loc_for_io())
+        return common.sanitize_url(track.get_tag_raw('__loc'))
+providers.register('tag-formatting', LocationTagFormatter())
 
-        return gfile.get_basename()
-providers.register('tag_formatting', FilenameTagFormatter())
+class CommentTagFormatter(TagFormatter):
+    """
+        A formatter for comments embedded in tracks
+    """
+    def __init__(self):
+        TagFormatter.__init__(self, 'comment')
+
+    def format(self, track, parameters):
+        """
+            Formats a raw tag value
+
+            :param track: the track to get the tag from
+            :type track: :class:`xl.trax.Track`
+            :param parameters: whether to keep newlines,
+                possible values for "newlines" are:
+
+                * keep: do not strip newlines (default)
+                * strip: strip newlines
+            :type parameters: dictionary
+            :returns: the formatted value
+            :rtype: string
+        """
+        value = track.get_tag_disk(self.name)
+
+        if not value:
+            return ''
+
+        value = '\n\n'.join(value)
+        newlines = parameters.get('newlines', 'keep')
+
+        return self.format_value(value, newlines)
+
+    @staticmethod
+    def format_value(value, newlines='keep'):
+        """
+            Formats a comment value
+
+            :param value: the comment text
+            :type value: string
+            :param newlines: whether to keep newlines,
+                possible values are:
+
+                * keep: do not strip newlines (default)
+                * strip: strip newlines
+
+            :type newlines: string
+            :returns: the formatted value
+            :rtype: string
+        """
+        if newlines == 'strip':
+            value = ' '.join(value.splitlines())
+
+        return value
+providers.register('tag-formatting', CommentTagFormatter())
 
 # vim: et sts=4 sw=4
