@@ -31,24 +31,48 @@
 #
 # Also takes care of parsing commandline options.
 
-import os
-import sys
 import logging
 import logging.handlers
+import os
+import sys
 
+try:
+    import gio
+except ImportError:
+    # on Win32 using the GStreamer SDK, requires import of
+    # pygtk first
+    import pygtk
+    pygtk.require('2.0')
+    import gio
+
+from xl import common, xdg
 from xl.nls import gettext as _
-from xl import common, xdg, event
 
 # placeholder, - xl.version can be slow to import, which would slow down
 # cli args. Thus we import __version__ later.
 __version__ = None
 
-# initiate the logger. logger params are set later
-logger = logging.getLogger(__name__)
+logger = None
 
 class Exaile(object):
     _exaile = None
 
+    def __get_player(self):
+        raise DeprecationWarning('Using exaile.player is deprecated: '
+                                 'import xl.player.PLAYER instead.')
+
+    def __get_queue(self):
+        raise DeprecationWarning('Using exaile.queue is deprecated: '
+                                 'import xl.player.QUEUE instead.')
+
+    def __get_lyrics(self):
+        raise DeprecationWarning('Using exaile.lyrics is deprecated: '
+                                 'import xl.lyrics.MANAGER instead.')
+
+    player = property(__get_player)
+    queue = property(__get_queue)
+    lyrics = property(__get_lyrics)
+
     def __init__(self):
         """
             Initializes Exaile.
@@ -64,12 +88,28 @@ class Exaile(object):
         if self.options.UseDataDir:
             xdg.data_dirs.insert(1, self.options.UseDataDir)
 
+        # this is useful on Win32, because you cannot set these directories
+        # via environment variables
+        if self.options.UseAllDataDir:
+            xdg.data_home = self.options.UseAllDataDir
+            xdg.data_dirs.insert(0, xdg.data_home)
+            xdg.config_home = self.options.UseAllDataDir
+            xdg.config_dirs.insert(0, xdg.config_home)
+            xdg.cache_home = self.options.UseAllDataDir
+            
         xdg._make_missing_dirs()
 
+        # Make event debug imply debug
+        if self.options.DebugEvent:
+            self.options.Debug = True
+
         self.setup_logging()
         global logger
         logger = logging.getLogger(__name__)
 
+        # Late import ensures xl.event uses correct logger
+        from xl import event
+
         if self.options.EventFilter:
             event.EVENT_MANAGER.logger_filter = self.options.EventFilter
             self.options.DebugEvent = True
@@ -78,7 +118,6 @@ class Exaile(object):
             event.EVENT_MANAGER.use_logger = True
             self.options.Debug = True
 
-
         # initial mainloop setup. The actual loop is started later,
         # if necessary
         self.mainloop_init()
@@ -107,7 +146,7 @@ class Exaile(object):
             xldbus.run_commands(self.options, self.dbus)
 
         #connect dbus signals
-        if self.options.Dbus:
+        if self.options.StartGui and self.options.Dbus:
             self.dbus._connect_signals()
 
         # On SIGTERM, quit normally.
@@ -133,9 +172,16 @@ class Exaile(object):
             common.log_exception(log=logger)
             exit(1)
 
-        # Splash screen
+        splash = None
+
         if self.options.StartGui:
-            self.__show_splash()
+            from xl import settings
+
+            if settings.get_option('gui/use_splash', True):
+                from xlgui.widgets.info import Splash
+
+                splash = Splash()
+                splash.show()
 
         firstrun = settings.get_option("general/first_run", True)
 
@@ -155,13 +201,16 @@ class Exaile(object):
         from xl.migrations.settings import rating
         rating.migrate()
 
+        # Migrate builtin OSD to plugin
+        from xl.migrations.settings import osd
+        osd.migrate()
+
         # Initialize plugin manager
+        from xl import plugins
         if not self.options.SafeMode:
-            from xl import plugins
             logger.info("Loading plugins...")
             self.plugins = plugins.PluginsManager(self)
         else:
-            from xl import plugins
             logger.info("Safe mode enabled, not loading plugins.")
             self.plugins = plugins.PluginsManager(self, load=False)
 
@@ -175,11 +224,10 @@ class Exaile(object):
             common.log_exception(log=logger)
             exit(1)
 
+        from xl import event
         # Set up the player and playback queue
         from xl import player
-        self.player = player.PLAYER
-        self.queue = player.QUEUE
-        event.log_event("player_loaded", self, None)
+        event.log_event("player_loaded", player.PLAYER, None)
 
         # Initalize playlist manager
         from xl import playlist
@@ -192,7 +240,7 @@ class Exaile(object):
 
         # Initialize dynamic playlist support
         from xl import dynamic
-        self.dynamic = dynamic.DynamicManager(self.collection)
+        dynamic.MANAGER.collection = self.collection
 
         # Initalize device manager
         logger.info("Loading devices...")
@@ -213,10 +261,6 @@ class Exaile(object):
         self.stations = playlist.PlaylistManager('radio_stations')
         self.radio = radio.RadioManager()
 
-        # Initialize lyrics manager
-        from xl import lyrics
-        self.lyrics = lyrics.LyricsManager()
-
         self.gui = None
         # Setup GUI
         if self.options.StartGui:
@@ -224,15 +268,11 @@ class Exaile(object):
 
             import xlgui
             self.gui = xlgui.Main(self)
-            if self.options.StartMinimized:
-                self.gui.main.window.iconify()
             self.gui.main.window.show_all()
-
-            import glib
-            if self.splash is not None:
-                glib.idle_add(self.splash.destroy)
             event.log_event("gui_loaded", self, None)
 
+            if splash is not None:
+                splash.destroy()
 
         restore = True
 
@@ -240,7 +280,7 @@ class Exaile(object):
             # Find out if the user just passed in a list of songs
             # TODO: find a better place to put this
             # using arg[2:] because arg[1:] will include --startgui
-            args = [ os.path.abspath(arg) for arg in self.args ]
+            args = [ gio.File(arg).get_uri() for arg in self.args ]
             if len(args) > 0:
                 restore = False
                 self.gui.open_uri(args[0], play=True)
@@ -248,7 +288,7 @@ class Exaile(object):
                 self.gui.open_uri(arg)
 
         if restore:
-            self.queue._restore_player_state(
+            player.QUEUE._restore_player_state(
                     os.path.join(xdg.get_data_dir(), 'player.state'))
 
         if firstrun:
@@ -263,20 +303,30 @@ class Exaile(object):
         """
             Displays the splash screen
         """
-        import xlgui
         from xl import settings
-        self.splash = xlgui.show_splash(
-            show=settings.get_option('gui/use_splash', True))
+
+        if not settings.get_option('gui/use_splash', True):
+            return
+
+        from xlgui.widgets.info import Splash
+
+        splash = Splash()
+        splash.show()
 
     def setup_logging(self):
         console_format = "%(levelname)-8s: %(message)s"
         loglevel = logging.INFO
+
+        if self.options.DebugThreads:
+            console_format = "%(threadName)s:" + console_format
+
         if self.options.Debug:
             loglevel = logging.DEBUG
             console_format = "%(asctime)s,%(msecs)3d:" + console_format
             console_format += " (%(name)s)" # add module name
         elif self.options.Quiet:
             loglevel = logging.WARNING
+
         # Logfile level should always be INFO or higher
         if self.options.Quiet:
             logfilelevel = logging.INFO
@@ -289,6 +339,35 @@ class Exaile(object):
         logging.basicConfig(level=loglevel, format=console_format,
                 datefmt=datefmt)
 
+        class FilterLogger(logging.Logger):
+            class Filter(logging.Filter):
+                def filter(self, record):
+                    pass_record = True
+
+                    if FilterLogger.module is not None:
+                        pass_record = record.name == self.module
+
+                    if FilterLogger.level != logging.NOTSET and pass_record:
+                        pass_record = record.levelno == self.level
+
+                    return pass_record
+
+            module = None
+            level = logging.NOTSET
+
+            def __init__(self, name):
+                logging.Logger.__init__(self, name)
+
+                log_filter = self.Filter(name)
+                log_filter.module = FilterLogger.module
+                log_filter.level = FilterLogger.level
+                self.addFilter(log_filter)
+
+        FilterLogger.module = self.options.ModuleFilter
+        if self.options.LevelFilter is not None:
+            FilterLogger.level = getattr(logging, self.options.LevelFilter)
+        logging.setLoggerClass(FilterLogger)
+
         # Create log directory
         logdir = os.path.join(xdg.get_data_dir(), 'logs')
         if not os.path.exists(logdir):
@@ -345,6 +424,8 @@ class Exaile(object):
                 default=False, help=_("Stop playback"))
         group.add_option("-a", "--play", dest="Play", action="store_true",
                 default=False, help=_("Play"))
+        group.add_option("-u", "--pause", dest="Pause", action="store_true",
+                default=False, help=_("Pause"))
         group.add_option("-t", "--play-pause", dest="PlayPause",
                 action="store_true", default=False,
                 help=_("Pause or resume playback"))
@@ -355,14 +436,29 @@ class Exaile(object):
 
         group = OptionGroup(p, _('Collection Options'))
         group.add_option("--add", dest="Add", action="store",
-                # TRANSLATORS: Meta variable for --add
+                # TRANSLATORS: Meta variable for --add and --export-playlist
                 metavar=_("LOCATION"), help=_("Add tracks from LOCATION "
                                               "to the collection"))
         p.add_option_group(group)
 
+        group = OptionGroup(p, _('Playlist Options'))
+        group.add_option("--export-playlist", dest="ExportPlaylist",
+                         # TRANSLATORS: Meta variable for --add and --export-playlist
+                         action="store", metavar=_("LOCATION"),
+                         help=_('Exports the current playlist to LOCATION'))
+        p.add_option_group(group)
+
         group = OptionGroup(p, _('Track Options'))
         group.add_option("-q", "--query", dest="Query", action="store_true",
                 default=False, help=_("Query player"))
+        group.add_option("--format-query", dest="FormatQuery",
+                         # TRANSLATORS: Meta variable for --format-query
+                         action="store", metavar=_('FORMAT'),
+                         help=_('Retrieves the current playback state and track information as FORMAT'))
+        group.add_option("--format-query-tags", dest="FormatQueryTags",
+                         # TRANSLATORS: Meta variable for --format-query-tags
+                         action="store", metavar=_('TAGS'),
+                         help=_('TAGS to retrieve from the current track, use with --format-query'))
         group.add_option("--gui-query", dest="GuiQuery",
                 action="store_true", default=False,
                 help=_("Show a popup with data of the current track"))
@@ -435,14 +531,26 @@ class Exaile(object):
         group = OptionGroup(p, _('Development/Debug Options'))
         group.add_option("--datadir", dest="UseDataDir",
                 metavar=_('DIRECTORY'), help=_("Set data directory"))
+        group.add_option("--all-data-dir", dest="UseAllDataDir",
+                metavar=_('DIRECTORY'), help=_("Set data and config directory"))
+        group.add_option("--modulefilter", dest="ModuleFilter",
+                action="store", type="string", metavar=_('MODULE'),
+                help=_('Limit log output to MODULE'))
+        group.add_option("--levelfilter", dest="LevelFilter",
+                action="store", metavar=_('LEVEL'),
+                help=_('Limit log output to LEVEL'),
+                choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'])
         group.add_option("--debug", dest="Debug", action="store_true",
                 default=False, help=_("Show debugging output"))
         group.add_option("--eventdebug", dest="DebugEvent",
                 action="store_true", default=False, help=_("Enable debugging"
                 " of xl.event. Generates LOTS of output"))
+        group.add_option("--threaddebug", dest="DebugThreads",
+                action="store_true", default=False, help=_("Add thread name"
+                " to logging messages."))
         group.add_option("--eventfilter", dest="EventFilter",
                 action='store', type='string', metavar=_('TYPE'),
-                help=_("Limits event debug to output of TYPE"))
+                help=_("Limit xl.event debug to output of TYPE"))
         group.add_option("--quiet", dest="Quiet", action="store_true",
                 default=False, help=_("Reduce level of output"))
         group.add_option('--startgui', dest='StartGui', action='store_true',
@@ -457,11 +565,7 @@ class Exaile(object):
 
     def version(self):
         from xl.version import __version__
-        print r"""   ____          _ __    __
-  / __/_ _____ _(_) /__ / /
- / _/ \ \ / _ `/ / / -_)_/
-/___//_\_\\_,_/_/_/\__(_)   v%s
-"""%__version__
+        print "Exaile", __version__
         exit()
 
     def _add_default_playlists(self):
@@ -490,17 +594,18 @@ class Exaile(object):
             self.smart_playlists.save_playlist(pl, overwrite=True)
 
     def mainloop_init(self):
-        import glib, gobject
+        import gobject
+
         gobject.threads_init()
+
         if self.options.Dbus:
             import dbus, dbus.mainloop.glib
             dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
             dbus.mainloop.glib.threads_init()
             dbus.mainloop.glib.gthreads_init()
-        if self.options.StartGui:
-            import gtk
-            gtk.gdk.threads_init()
-        else:
+
+        if not self.options.StartGui:
+            import glib
             loop = glib.MainLoop()
             context = loop.get_context()
             self.__mainloop(context)
@@ -540,11 +645,12 @@ class Exaile(object):
                 except:
                     pass
 
+        from xl import event
         # this event should be used by modules that dont need
         # to be saved in any particular order. modules that might be
         # touched by events triggered here should be added statically
         # below.
-        event.log_event("quit_application", self, None, async=False)
+        event.log_event("quit_application", self, None)
 
         logger.info("Saving state...")
         self.plugins.save_enabled()
@@ -562,14 +668,14 @@ class Exaile(object):
         self.stations.save_order()
 
         # save player, queue
-        self.queue._save_player_state(
+        from xl import player
+        player.QUEUE._save_player_state(
                 os.path.join(xdg.get_data_dir(), 'player.state') )
-        self.queue.save_to_location(
+        player.QUEUE.save_to_location(
                 os.path.join(xdg.get_data_dir(), 'queue.state') )
-        self.player.stop()
+        player.PLAYER.stop()
 
         from xl import settings
-
         settings.MANAGER.save()
 
         if restart:
