@@ -28,9 +28,10 @@ import logging
 import threading
 import time
 
+import glib
 import pygst
 pygst.require('0.10')
-import gst, gobject, glib
+import gst
 
 from xl.nls import gettext as _
 from xl import event, settings, common
@@ -39,11 +40,16 @@ from xl.player import _base, pipe
 logger = logging.getLogger(__name__)
 
 class UnifiedPlayer(_base.ExailePlayer):
-    def __init__(self):
-        _base.ExailePlayer.__init__(self)
+    def __init__(self, name):
+        self.caps = None
+        self.adder = None
+        self.audio_queue = None
+        _base.ExailePlayer.__init__(self, name)
         self._current_stream = 1
         self._timer_id = 0
+        self.streams = [None, None]
 
+    def _setup_pipe(self):
         # have to fix the caps because gst cant deal with having them change.
         # TODO: make this a preference and/or autodetect optimal based on the
         #   output device - if its a 48000hz-native chip we dont want to send it
@@ -56,16 +62,19 @@ class UnifiedPlayer(_base.ExailePlayer):
                 "width=(int)16, "
                 "depth=(int)16, "
                 "rate=(int)44100, "
-                "channels=(int)2")
-        self.pipe = gst.Pipeline()
+                "channels=(int)2"
+                )
+        self._pipe = gst.Pipeline()
         self.adder = gst.element_factory_make("adder")
         self.audio_queue = gst.element_factory_make("queue")
-
-        self.streams = [None, None]
-
         self._load_queue_values()
-        self._setup_pipeline()
-        self.setup_bus()
+        self._pipe.add(
+                self.adder,
+                self.audio_queue,
+                self._mainbin
+                )
+        self.adder.link(self.audio_queue)
+        self.audio_queue.link(self._mainbin)
 
     def _load_queue_values(self):
         # queue defaults to 1 second of audio data, however this
@@ -76,23 +85,16 @@ class UnifiedPlayer(_base.ExailePlayer):
         # collections are on slower media can increase it to preserve
         # gapless, at the expense of UI lag.
         self.audio_queue.set_property("max-size-time",
-                settings.get_option("player/queue_duration", 1000000))
-
-    def _setup_pipeline(self):
-        self.pipe.add(
-                self.adder,
-                self.audio_queue,
-                self.mainbin
-                )
-        self.adder.link(self.audio_queue)
-        self.audio_queue.link(self.mainbin)
+                settings.get_option("%s/queue_duration" % self._name, 1000000))
 
     def _on_drained(self, dec, stream):
         logger.debug("%s drained"%stream.get_name())
         #if stream.track != self.current:
         #    return
-        if not settings.get_option("player/crossfading", False):
-            tr = self._queue.next(player=False)
+        if not settings.get_option("%s/crossfading" % self._name, False):
+            tr = None
+            if settings.get_option("%s/auto_advance" % self._name, True):
+                tr = self.queue.next(autoplay=False)
             self.unlink_stream(stream)
             if tr is None:
                 self.stop()
@@ -101,45 +103,10 @@ class UnifiedPlayer(_base.ExailePlayer):
         else:
             self.unlink_stream(stream)
 
-    def setup_bus(self):
-        """
-            setup the gstreamer message bus and callacks
-        """
-        self.bus = self.pipe.get_bus()
-        self.bus.add_signal_watch()
-        self.bus.enable_sync_message_emission()
-        self.bus.connect('message', self.on_message)
-
-    def on_message(self, bus, message, reading_tag = False):
-        if message.type == gst.MESSAGE_EOS and not self.is_paused():
-            logger.warning("EOS: ", message)
-        elif message.type == gst.MESSAGE_TAG and self.tag_func:
-            self.tag_func(message.parse_tag())
-            if not self.current.get_tag_raw('__length'):
-                try:
-                    duration = float(self.mainbin.query_duration(
-                        gst.FORMAT_TIME, None)[0])/1000000000
-                    if duration > 0:
-                        self.current.set_tag_raw('__length', duration)
-                except gst.QueryError:
-                    logger.debug("Couldn't query duration via GStreamer")
-        elif message.type == gst.MESSAGE_ERROR:
-            logger.error("%s %s" %(message, dir(message)) )
-            a = message.parse_error()[0]
-            self._on_playback_error(a.message)
-        return True
-
     def _get_current(self):
         if self.streams[self._current_stream]:
             return self.streams[self._current_stream].get_current()
 
-    def _get_gst_state(self):
-        """
-            Returns the raw GStreamer state
-        """
-        return self.pipe.get_state(timeout=50*gst.MSECOND)[1]
-
-
     def get_position(self):
         try:
             return self.streams[self._current_stream].get_position()
@@ -153,7 +120,7 @@ class UnifiedPlayer(_base.ExailePlayer):
 
         playing = self.is_playing()
 
-        logger.debug("Attmepting to play \"%s\""%track)
+        logger.debug("%s: Attempting to play \"%s\""% (self._name, track))
         next = 1-self._current_stream
 
         if self.streams[next]:
@@ -163,23 +130,23 @@ class UnifiedPlayer(_base.ExailePlayer):
         duration = 0
 
         if user:
-            if settings.get_option("player/user_fade_enabled", False):
+            if settings.get_option("%s/user_fade_enabled" % self._name, False):
                 fading = True
-                duration = settings.get_option("player/user_fade", 1000)
+                duration = settings.get_option("%s/user_fade" % self._name, 1000)
             else:
                 self.unlink_stream(self.streams[self._current_stream])
         else:
-            if settings.get_option("player/crossfading", False):
+            if settings.get_option("%s/crossfading" % self._name, False):
                 fading = True
                 duration = settings.get_option(
-                        "player/crossfade_duration", 3000)
+                        "%s/crossfade_duration" % self._name, 3000)
             else:
                 self.unlink_stream(self.streams[self._current_stream])
 
         if not playing:
-            event.log_event_sync('playback_reconfigure_bins', self, None)
+            event.log_event('playback_reconfigure_bins', self, None)
 
-        self.streams[next] = AudioStream("Stream%s"%(next), caps=self.caps)
+        self.streams[next] = AudioStream("Stream%s"%(next), self, caps=self.caps)
         self.streams[next].dec.connect("drained", self._on_drained,
                 self.streams[next])
 
@@ -189,10 +156,10 @@ class UnifiedPlayer(_base.ExailePlayer):
         if fading:
             self.streams[next].set_volume(0)
 
-        self.pipe.set_state(gst.STATE_PLAYING)
+        self._pipe.set_state(gst.STATE_PLAYING)
         self.streams[next]._settle_flag = 1
         glib.idle_add(self.streams[next].set_state, gst.STATE_PLAYING)
-        glib.idle_add(self._set_state, self.pipe, gst.STATE_PLAYING)
+        glib.idle_add(self._set_state, self._pipe, gst.STATE_PLAYING)
 
         if fading:
             timeout = int(float(duration)/float(100))
@@ -202,7 +169,7 @@ class UnifiedPlayer(_base.ExailePlayer):
             if self.streams[self._current_stream]:
                 glib.timeout_add(timeout, self._fade_stream,
                         self.streams[self._current_stream], -1, True)
-            if settings.get_option("player/crossfading", False):
+            if settings.get_option("%s/crossfading" % self._name, False):
                 time = int(track.get_tag_raw("__length")*1000 - duration)
                 glib.timer_id = glib.timeout_add(time,
                         self._start_crossfade)
@@ -231,7 +198,9 @@ class UnifiedPlayer(_base.ExailePlayer):
         return 0.01 <= current <= 1
 
     def _start_crossfade(self, *args):
-        tr = self._queue.next(player=False)
+        tr = None
+        if settings.get_option("%s/auto_advance" % self._name, True):
+            tr = self.queue.next(autoplay=False)
         if tr is not None:
             self.play(tr, user=False)
         if self._timer_id:
@@ -247,9 +216,9 @@ class UnifiedPlayer(_base.ExailePlayer):
             glib.source_remove(self._timer_id)
         if not self.is_playing():
             return
-        if not settings.get_option("player/crossfading", False):
+        if not settings.get_option("%s/crossfading" % self._name, False):
             return
-        duration = settings.get_option("player/crossfade_duration", 3000)
+        duration = settings.get_option("%s/crossfade_duration" % self._name, 3000)
         time = int( self.current.get_tag_raw('__length')*1000 - \
                 (self.get_time()*1000 + duration) )
         if time < duration: # start crossfade now, we're late!
@@ -268,7 +237,7 @@ class UnifiedPlayer(_base.ExailePlayer):
                 pass
             glib.idle_add(stream.set_state, gst.STATE_NULL)
             try:
-                self.pipe.remove(stream)
+                self._pipe.remove(stream)
             except gst.RemoveError:
                 logger.debug("Failed to remove stream %s"%stream)
             if stream in self.streams:
@@ -282,7 +251,7 @@ class UnifiedPlayer(_base.ExailePlayer):
             return False
 
     def link_stream(self, stream, track):
-        self.pipe.add(stream)
+        self._pipe.add(stream)
         stream.link(self.adder)
         if not stream.set_track(track):
             logger.error("Failed to start playing \"%s\""%track)
@@ -291,48 +260,31 @@ class UnifiedPlayer(_base.ExailePlayer):
         return True
 
     @common.synchronized
-    def stop(self):
+    def _stop(self):
         """
             stop playback
         """
-        if self.is_playing() or self.is_paused():
-            current = self.current
-            self.pipe.set_state(gst.STATE_NULL)
-            for stream in self.streams:
-                self.unlink_stream(stream)
-            self._reset_crossfade_timer()
-            event.log_event('playback_player_end', self, current)
-            return True
-        return False
+        current = self.current
+        self._pipe.set_state(gst.STATE_NULL)
+        for stream in self.streams:
+            self.unlink_stream(stream)
+        self._reset_crossfade_timer()
+        return current
 
     @common.synchronized
-    def pause(self):
-        """
-            pause playback. DOES NOT TOGGLE
-        """
-        if self.is_playing():
-            self.pipe.set_state(gst.STATE_PAUSED)
-            self._reset_crossfade_timer()
-            event.log_event('playback_player_pause', self, self.current)
-            return True
-        return False
+    def _pause(self):
+        self._pipe.set_state(gst.STATE_PAUSED)
+        self._reset_crossfade_timer()
 
     @common.synchronized
-    def unpause(self):
-        """
-            unpause playback
-        """
-        if self.is_paused():
-            # gstreamer does not buffer paused network streams, so if the user
-            # is unpausing a stream, just restart playback
-            if not self.current.is_local():
-                self.pipe.set_state(gst.STATE_READY)
+    def _unpause(self):
+        # gstreamer does not buffer paused network streams, so if the user
+        # is unpausing a stream, just restart playback
+        if not self.current.is_local():
+            self._pipe.set_state(gst.STATE_READY)
 
-            self.pipe.set_state(gst.STATE_PLAYING)
-            self._reset_crossfade_timer()
-            event.log_event('playback_player_resume', self, self.current)
-            return True
-        return False
+        self._pipe.set_state(gst.STATE_PLAYING)
+        self._reset_crossfade_timer()
 
     @common.synchronized
     def seek(self, value):
@@ -342,9 +294,11 @@ class UnifiedPlayer(_base.ExailePlayer):
         self.streams[self._current_stream].seek(value)
         self._reset_crossfade_timer()
 
+        event.log_event('playback_seeked', self, value)
+
 
 class AudioStream(gst.Bin):
-    def __init__(self, name, caps=None):
+    def __init__(self, name, player, caps=None):
         gst.Bin.__init__(self, name)
         self.notify_id = None
         self.track = None
@@ -356,13 +310,13 @@ class AudioStream(gst.Bin):
         self._seek_event = threading.Event()
 
         self.caps = caps
-        self.setup_elems()
+        self.setup_elems(player)
 
-    def setup_elems(self):
+    def setup_elems(self, player):
         self.dec = gst.element_factory_make("uridecodebin")
         self.audioconv = gst.element_factory_make("audioconvert")
         self.audioresam = gst.element_factory_make("audioresample")
-        self.provided = pipe.ProviderBin("stream_element")
+        self.provided = pipe.ProviderBin(player, "stream_element")
         self.capsfilter = gst.element_factory_make("capsfilter")
         self.capsfilter.set_property("caps", self.caps)
         self.vol = gst.element_factory_make("volume")
@@ -402,7 +356,7 @@ class AudioStream(gst.Bin):
         if track.is_local():
             if not track.exists():
                 logger.error("File does not exist: %s" %
-                        track.get_loc_for_display())
+                        track.get_loc_for_io())
                 return False
 
         self.track = track
@@ -528,7 +482,7 @@ class AudioStream(gst.Bin):
             seek to the given position in the current stream
         """
         if self._settle_flag == 1:
-            event.add_callback(self._seek_delayed, "stream_settled")
+            event.add_callback(self._seek_delayed, "stream_settled", self)
             self._seek_event.clear()
             self._seek_event.wait()
 
@@ -539,8 +493,6 @@ class AudioStream(gst.Bin):
 
         self.vol.send_event(seekevent)
 
-        self.last_seek_pos = value
-
     def _seek_delayed(self, type, object, value):
         """
             internal code used if seek is called before the stream is ready
