@@ -24,6 +24,10 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+    Central storage of application and user settings
+"""
+
 from __future__ import with_statement
 from ConfigParser import (
     RawConfigParser,
@@ -38,7 +42,7 @@ import glib
 logger = logging.getLogger(__name__)
 
 from xl import event, xdg
-from xl.common import VersionError
+from xl.common import VersionError, glib_wait_seconds
 from xl.nls import gettext as _
 
 TYPE_MAPPING = {
@@ -63,7 +67,8 @@ class SettingsManager(RawConfigParser):
     def __init__(self, location=None, default_location=None):
         """
             Sets up the settings manager. Expects a location
-            to a file where settings will be stored.
+            to a file where settings will be stored. Also sets up
+            periodic saves to disk.
 
             :param location: the location to save the settings to,
                 settings will never be stored if this is None
@@ -99,10 +104,11 @@ class SettingsManager(RawConfigParser):
 
         # save settings every 30 seconds
         if location is not None:
-            glib.timeout_add_seconds(30, self._timeout_save)
+            self._timeout_save()
 
+    @glib_wait_seconds(30)
     def _timeout_save(self):
-        #logger.debug("Requesting save from timeout...")
+        """Save every 30 seconds"""
         self.save()
         return True
 
@@ -128,11 +134,12 @@ class SettingsManager(RawConfigParser):
 
     def set_option(self, option, value):
         """
-            Set an option (in section/key syntax) to the specified value.
+            Set an option (in ``section/key`` syntax) to the specified value
 
             :param option: the full path to an option
             :type option: string
             :param value: the value the option should be assigned
+            :type value: any
         """
         value = self._val_to_str(value)
         splitvals = option.split('/')
@@ -153,10 +160,14 @@ class SettingsManager(RawConfigParser):
 
     def get_option(self, option, default=None):
         """
-            Get the value of an option (in section/key syntax), returning
-            default if the key does not exist yet.
+            Get the value of an option (in ``section/key`` syntax),
+            returning *default* if the key does not exist yet
 
-            :returns: the option value or default
+            :param option: the full path to an option
+            :type option: string
+            :param default: a default value to use as fallback
+            :type default: any
+            :returns: the option value or *default*
             :rtype: any
         """
         splitvals = option.split('/')
@@ -189,7 +200,7 @@ class SettingsManager(RawConfigParser):
 
     def remove_option(self, option):
         """
-            Removes an option (in section/key syntax),
+            Removes an option (in ``section/key`` syntax),
             thus will not be saved anymore
 
             :param option: the option path
@@ -254,7 +265,10 @@ class SettingsManager(RawConfigParser):
                 if value != 'True':
                     return False
 
-            value = TYPE_MAPPING[kind](value)
+            try:
+                value = TYPE_MAPPING[kind](value)
+            except:
+                pass
 
             return value
         else:
