@@ -27,25 +27,40 @@
 import gio
 from xl import metadata, settings
 from xl.trax.track import Track
+from xl.trax.search import search_tracks, TracksMatcher
 
 
-def is_valid_track(loc):
+def is_valid_track(location):
     """
-        Returns whether the file at loc is a valid track,
-        right now determines based on file extension
+        Returns whether the file at the given location is a valid track
+
+        :param location: the location to check
+        :type location: string
+        :returns: whether the file is a valid track
+        :rtype: boolean
     """
-    extension = gio.File(loc).get_basename().split(".")[-1]
+    extension = gio.File(location).get_basename().split(".")[-1]
     return extension.lower() in metadata.formats
 
 def get_uris_from_tracks(tracks):
     """
         Returns all URIs for tracks
+
+        :param tracks: the tracks to retrieve the URIs from
+        :type tracks: list of :class:`xl.trax.Track`
+        :returns: the uris
+        :rtype: list of string
     """
     return [track.get_loc_for_io() for track in tracks]
 
 def get_tracks_from_uri(uri):
     """
         Returns all valid tracks located at uri
+        
+        :param uri: the uri to retrieve the tracks from
+        :type uri: string
+        :returns: the retrieved tracks
+        :rtype: list of :class:`xl.trax.Track`
     """
     tracks = []
 
@@ -54,7 +69,7 @@ def get_tracks_from_uri(uri):
     # otherwise be terribly slow.
     # TODO: move uri definition somewhere more common for easy reuse?
 
-    if gloc.get_uri_scheme() in ('http', 'mms'):
+    if gloc.get_uri_scheme() in ('http', 'mms', 'cdda'):
         return [Track(uri)]
 
     file_type = gloc.query_info("standard::type").get_file_type()
@@ -74,12 +89,16 @@ def sort_tracks(fields, iter, trackfunc=None, reverse=False):
     """
         Sorts tracks.
 
-        :param fields: An iterable of tag names
-                       to sort by.
-        :param iter: The iterable to sort. Must be either Track objects, or
-                    trackfunc must be specified.
-        :param trackfunc: Function to get a Track from an item in the iter.
-        :param reverse: Whether to sort in reverse.
+        :param fields: tag names to sort by
+        :type fields: iterable
+        :param iter: the tracks to sort,
+            alternatively use *trackfunc*
+        :type iter: iterable
+        :param trackfunc: function to get a *Track*
+            from an item in the iterable
+        :type trackfunc: function or None
+        :param reverse: whether to sort in reversed order
+        :type reverse: boolean
     """
     fields = list(fields) # we need the index method
     artist_compilations = True
@@ -89,7 +108,6 @@ def sort_tracks(fields, iter, trackfunc=None, reverse=False):
         artist_compilations=artist_compilations) for field in fields]
     return sorted(iter, key=keyfunc, reverse=reverse)
 
-
 def sort_result_tracks(fields, trackiter, reverse=False):
     """
         Sorts SearchResultTracks, ie. the output from a search.
@@ -105,14 +123,15 @@ def get_rating_from_tracks(tracks):
         rating. Same goes if the amount of tracks
         is 0 or more than the internal limit.
 
-        :param tracks: an iterable containing objects
-            of type :class:`xl.trax.Track`
+        :param tracks: the tracks to retrieve the rating from
+        :type tracks: iterable
     """
     if len(tracks) < 1:
         return 0
 
-    if len(tracks) > settings.get_option('rating/tracks_limit', 100):
-        return 0
+# TODO: still needed?
+#    if len(tracks) > settings.get_option('rating/tracks_limit', 100):
+#        return 0
 
     rating = tracks[0].get_rating()
 
@@ -121,3 +140,18 @@ def get_rating_from_tracks(tracks):
             return 0
 
     return rating
+
+
+
+def get_album_tracks(tracksiter, track, artist_compilations=True):
+    """
+        Get any tracks from the given iterable that appear to be part of
+        the same album as track. If track is in the iterable, it will be
+        included in the result. If there is insufficient information to
+        determine the album, the empty list will be returned, even if the
+        track is in the iterable.
+    """
+    if not all(track.get_tag_raw(t) for t in ['artist', 'album']):
+        return []
+    matchers = map(lambda t: TracksMatcher(track.get_tag_search(t, artist_compilations=artist_compilations)), ['artist', 'album'])
+    return (r.track for r in search_tracks(tracksiter, matchers))
