@@ -24,6 +24,14 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+    D-Bus interface for playback control, data query and others
+
+    Access through the ``/org/exaile/Exaile`` object which
+    implements the ``org.exaile.Exaile`` interface
+"""
+
+from collections import namedtuple
 import logging
 from optparse import OptionParser
 import os
@@ -74,15 +82,15 @@ def check_exit(options, args):
                 # This enables:    find PATH -name *.mp3 | exaile -
                 if args[0] == '-':
                     args = sys.stdin.read().split('\n')
-                args = [ os.path.abspath(arg) for arg in args ]
+                args = [ gio.File(arg).get_uri() for arg in args ]
                 iface.Enqueue(args)
 
     if not iface:
         for command in ['GetArtist', 'GetTitle', 'GetAlbum', 'GetLength',
                 'GetRating', 'SetRating', 'IncreaseVolume', 'DecreaseVolume',
-                'Play', 'Stop', 'Next', 'Prev', 'PlayPause',
+                'Play', 'Pause', 'Stop', 'Next', 'Prev', 'PlayPause',
                 'StopAfterCurrent', 'GuiToggleVisible', 'CurrentPosition',
-                'CurrentProgress', 'GetVolume', 'Query']:
+                'CurrentProgress', 'GetVolume', 'Query', 'FormatQuery']:
             if getattr(options, command):
                 return "command"
         return "continue"
@@ -111,54 +119,58 @@ def run_commands(options, iface):
                 print value
             comm = True
 
-    modify_commands = [
+    argument_commands = (
+        'IncreaseVolume',
+        'DecreaseVolume',
         'SetRating',
-        'Add'
-    ]
+        'Add',
+        'ExportPlaylist'
+    )
 
-    for command in modify_commands:
+    for command in argument_commands:
         argument = getattr(options, command)
         if argument is not None:
-            getattr(iface, command)(argument)
+            if command in ('IncreaseVolume', 'DecreaseVolume'):
+                iface.ChangeVolume(argument if command == 'IncreaseVolume' else -argument)
+            else:
+                print getattr(iface, command)(argument)
+
             comm = True
 
-    volume_commands = (
-            'IncreaseVolume',
-            'DecreaseVolume',
-            )
-
-    for command in volume_commands:
-        value = getattr(options, command)
-        if value:
-            if command == 'DecreaseVolume': value = -value
-            iface.ChangeVolume(value)
+    # Special handling for FormatQuery & FormatQueryTags
+    format = options.FormatQuery
+    if format is not None:
+        print iface.FormatQuery(format, options.FormatQueryTags or 'title,artist,album,__length')
+        comm = True
 
     run_commands = (
-            'Play',
-            'Stop',
-            'Next',
-            'Prev',
-            'PlayPause',
-            'StopAfterCurrent',
-            'GuiToggleVisible',
-            'ToggleMute'
-            )
+        'Play',
+        'Pause',
+        'Stop',
+        'Next',
+        'Prev',
+        'PlayPause',
+        'StopAfterCurrent',
+        'GuiToggleVisible',
+        'ToggleMute'
+    )
+
     for command in run_commands:
         if getattr(options, command):
             getattr(iface, command)()
             comm = True
 
     query_commands = (
-            'CurrentPosition',
-            'CurrentProgress',
-            'GetRating',
-            'GetVolume',
-            'Query',
-            )
+        'CurrentPosition',
+        'CurrentProgress',
+        'GetRating',
+        'GetVolume',
+        'Query'
+    )
 
     for command in query_commands:
         if getattr(options, command):
-            print getattr(iface, command)()
+            print getattr(iface, command)(argument)
             comm = True
 
     to_implement = (
@@ -172,6 +184,11 @@ def run_commands(options, iface):
     if not comm:
         iface.GuiToggleVisible()
 
+PlaybackStatus = namedtuple(
+    'PlaybackStatus',
+    'state progress position current'
+)
+
 class DbusManager(dbus.service.Object):
     """
         The dbus interface object for Exaile
@@ -210,20 +227,31 @@ class DbusManager(dbus.service.Object):
     def TestService(self, arg):
         """
             Just test the dbus object
+
+            :param arg: anything
         """
         logger.debug(arg)
 
     @dbus.service.method('org.exaile.Exaile', None, 'b')
     def IsPlaying(self):
         """
-            Returns True if Exaile is playing (or paused), False if it's not
+            Determines if Exaile is playing / paused or not
+
+            :returns: whether Exaile is playing / paused
+            :rtype: boolean
         """
-        return bool(self.exaile.player.current)
+        from xl import player
+        return bool(not player.PLAYER.is_stopped())
 
     @dbus.service.method('org.exaile.Exaile', 's', 's')
     def GetTrackAttr(self, attr):
         """
-            Returns an attribute of a track
+            Returns the value of a track tag
+
+            :param attr: a track tag
+            :type attr: string
+            :returns: the tag value
+            :rtype: string
         """
         from xl import player
         try:
@@ -238,7 +266,12 @@ class DbusManager(dbus.service.Object):
     @dbus.service.method('org.exaile.Exaile', 'sv')
     def SetTrackAttr(self, attr, value):
         """
-            Sets rating of a track
+            Sets the value of a track tag
+
+            :param attr: a track tag
+            :type attr: string
+            :param value: the tag value
+            :type value: any
         """
         from xl import player
         try:
@@ -250,6 +283,9 @@ class DbusManager(dbus.service.Object):
     def GetRating(self):
         """
             Returns the current track's rating
+
+            :returns: the rating
+            :rtype: int
         """
         try:
             rating = int(float(self.GetTrackAttr('__rating')))
@@ -262,6 +298,9 @@ class DbusManager(dbus.service.Object):
     def SetRating(self, value):
         """
             Sets the current track's rating
+
+            :param value: the new rating
+            :type value: int
         """
         self.SetTrackAttr('__rating', value)
         event.log_event('rating_changed', self, value)
@@ -270,6 +309,9 @@ class DbusManager(dbus.service.Object):
     def ChangeVolume(self, value):
         """
             Changes volume by the specified amount (in percent, can be negative)
+
+            :param value: the new volume
+            :type value: int
         """
         from xl import player
         player.PLAYER.set_volume(player.PLAYER.get_volume() + value)
@@ -292,6 +334,9 @@ class DbusManager(dbus.service.Object):
     def Seek(self, value):
         """
             Seeks to the given position in seconds
+
+            :param value: the position in seconds
+            :type value: int
         """
         from xl import player
         player.PLAYER.seek(value)
@@ -328,13 +373,21 @@ class DbusManager(dbus.service.Object):
         from xl import player
         player.QUEUE.play()
 
+    @dbus.service.method('org.exaile.Exaile')
+    def Pause(self):
+        """
+            Starts playback
+        """
+        from xl import player
+        player.PLAYER.pause()
+
     @dbus.service.method('org.exaile.Exaile')
     def PlayPause(self):
         """
             Toggle Play or Pause
         """
         from xl import player
-        if player.PLAYER.get_state() == 'stopped':
+        if player.PLAYER.is_stopped():
             player.PLAYER.play(player.QUEUE.get_current())
         else:
             player.PLAYER.toggle_pause()
@@ -351,6 +404,9 @@ class DbusManager(dbus.service.Object):
     def CurrentProgress(self):
         """
             Returns the progress into the current track (in percent)
+
+            :returns: the current progress
+            :rtype: string
         """
         from xl import player
         progress = player.PLAYER.get_progress()
@@ -362,6 +418,9 @@ class DbusManager(dbus.service.Object):
     def CurrentPosition(self):
         """
             Returns the position inside the current track (as time)
+
+            :returns: the current position
+            :rtype: string
         """
         from xl import player
         progress = player.PLAYER.get_time()
@@ -371,41 +430,116 @@ class DbusManager(dbus.service.Object):
     def GetVolume(self):
         """
             Returns the current volume level (in percent)
+
+            :returns: the current volume
+            :rtype: string
         """
         from xl import player
         return str(player.PLAYER.get_volume())
 
+    def __get_playback_status(self, tags=["title", "artist", "album", "__length"]):
+        """
+            Retrieves the playback status
+
+            :param tags: tags to retrieve from the current track
+            :type tags: list
+            :returns: the playback status
+            :rtype: :class:`xl.xldbus.PlaybackStatus`
+        """
+        status = PlaybackStatus(
+            state='stopped',
+            current=None,
+            progress=0,
+            position=0
+        )
+
+        from xl import player
+
+        current_track = player.QUEUE.get_current()
+        progress = player.PLAYER.get_progress()
+
+        if current_track is not None and \
+           not player.PLAYER.is_stopped():
+            current = {}
+            # Make tags unique
+            tags = list(set(tags))
+
+            for tag in tags:
+                current[tag] = self.GetTrackAttr(tag)
+
+            # Special handling for internal tags
+            if "__length" in tags:
+                from xl.formatter import LengthTagFormatter
+                current["__length"] = LengthTagFormatter.format_value(self.GetTrackAttr('__length'))
+            
+            status = PlaybackStatus(
+                state=player.PLAYER.get_state(),
+                progress=self.CurrentProgress(),
+                position=self.CurrentPosition(),
+                current=current
+            )
+
+        return status
+
+
     @dbus.service.method('org.exaile.Exaile', None, 's')
     def Query(self):
         """
             Returns information about the currently playing track
-        """
-        from xl import player
-        current_track = player.QUEUE.get_current()
-        if current_track is None or player.PLAYER.is_stopped():
-            return _('Not playing.')
 
-        length = float(self.GetTrackAttr('__length') or 0)
-        length = '%d:%02d' % (length // 60, length % 60)
+            :returns: information about the current track
+            :rtype: string
+        """
+        status = self.__get_playback_status()
+
+        if status.current is None or status.state == "stopped":
+            return _('Not playing.')
 
         result = _('status: %(status)s, title: %(title)s, artist: %(artist)s,'
                    ' album: %(album)s, length: %(length)s,'
                    ' position: %(progress)s%% [%(position)s]') % {
-                         'status': player.PLAYER.get_state(),
-                         'title': self.GetTrackAttr('title'),
-                         'artist': self.GetTrackAttr('artist'),
-                         'album': self.GetTrackAttr('album'),
-                         'length': length,
-                         'progress': self.CurrentProgress(),
-                         'position': self.CurrentPosition(),
+                         'status': status.state,
+                         'title': status.current["title"],
+                         'artist': status.current["artist"],
+                         'album': status.current["album"],
+                         'length': status.current["__length"],
+                         'progress': status.progress,
+                         'position': status.position,
                      }
 
         return result
 
+    @dbus.service.method('org.exaile.Exaile', 'ss', 's')
+    def FormatQuery(self, format, tags):
+        """
+            Returns the current playback state including
+            information about the currently playing track
+
+            :param format: the desired output format (currently only "json")
+            :type format: string
+            :param tags: tags to retrieve from the current track
+            :type tags: string
+            :returns: the formatted information or an empty
+                      string if requesting an unknown format
+            :rtype: string
+        """
+        # TODO: Elaborate the use of providers here
+        if format == 'json':
+            import json
+
+            status = self.__get_playback_status(tags.split(","))
+
+            return json.dumps(status._asdict())
+
+        return ''
+
     @dbus.service.method('org.exaile.Exaile', None, 's')
     def GetVersion(self):
         """
             Returns the version of Exaile
+
+            :returns: the application version
+            :rtype: string
         """
         return self.exaile.get_version()
 
@@ -413,6 +547,9 @@ class DbusManager(dbus.service.Object):
     def PlayFile(self, filename):
         """
             Plays the specified file
+
+            :param filename: the path to start playing
+            :type filename: string
         """
         self.exaile.gui.open_uri(filename)
 
@@ -421,57 +558,56 @@ class DbusManager(dbus.service.Object):
         """
             Adds the tracks at the specified locations
             to the current playlist
+
+            :param locations: locations to enqueue
+            :type locations: iterable
         """
-        import xl.playlist
         from xl import player
-        from xl import trax   # do this here to avoid loading
-                              # settings when issuing dbus commands
-        # FIXME: Get rid of dependency on xlgui
-        #        by moving sorting column somewhere else
-        pl = self.exaile.gui.main.get_selected_playlist()
-        column, descending = pl.get_sort_by()
-        tracks = []
-        play_track = None
-        playlists = []
+        from xlgui import get_controller
+
+        controller = get_controller()
+
+        play = False
+
+        # Allows for playing first item
+        if player.PLAYER.is_stopped():
+            play = True
 
         for location in locations:
-            tracks = []
-
-            if xl.playlist.is_valid_playlist(location):
-                try:
-                    pl = xl.playlist.import_playlist(location)
-                    tracks = pl.get_tracks()
-                    continue
-                except xl.playlist.InvalidPlaylistTypeException:
-                    pass
-                except:
-                    traceback.print_exc()
-            else:
-                tracks = trax.get_tracks_from_uri(location)
-
-            if tracks:
-                tracks = trax.sort_tracks(['album', column], tracks, reverse=descending)
-                player.QUEUE.current_playlist.add_tracks(tracks)
-
-                if play_track is None:
-                    play_track = tracks[0]
-
-        if player.PLAYER.is_stopped() and play_track is not None:
-            pos = player.QUEUE.current_playlist.index(play_track)
-            player.QUEUE.current_playlist.set_current_pos(pos)
-            player.QUEUE.play()
+            controller.open_uri(location, play=play)
+            play = False
 
     @dbus.service.method('org.exaile.Exaile', 's')
     def Add(self, location):
         """
             Adds the tracks at the specified
             location to the collection
+
+            :param location: where to add tracks from
+            :type location: string
         """
         from xl import trax
 
         tracks = trax.get_tracks_from_uri(location)
         self.exaile.collection.add_tracks(tracks)
 
+    @dbus.service.method('org.exaile.Exaile', 's')
+    def ExportPlaylist(self, location):
+        """
+            Exports the current playlist
+            to the specified location
+
+            :param location: where to save the playlist at
+            :type location: string
+        """
+        from xl import player, playlist
+
+        if player.QUEUE.current_playlist is not None:
+            playlist.export_playlist(
+                player.QUEUE.current_playlist,
+                location
+            )
+
     @dbus.service.method('org.exaile.Exaile')
     def GuiToggleVisible(self):
         """
@@ -484,9 +620,12 @@ class DbusManager(dbus.service.Object):
         """
             Returns the data of the cover image of the playing track, or
             an empty string if there is no cover available.
+
+            :returns: the cover data
+            :rtype: binary data
         """
-        from xl import covers
-        cover = covers.MANAGER.get_cover(player.PLAYER.current)
+        from xl import covers, player
+        cover = covers.MANAGER.get_cover(player.PLAYER.current, set_only=True)
         if not cover:
             cover = ''
         return cover
@@ -495,6 +634,9 @@ class DbusManager(dbus.service.Object):
     def GetState(self):
         """
             Returns the surrent verbatim state (unlocalized)
+
+            :returns: the player state
+            :rtype: string
         """
         from xl import player
         return player.PLAYER.get_state()
