@@ -29,6 +29,7 @@ import os
 import os.path
 import tempfile
 import time
+import threading
 import traceback
 
 import cairo
@@ -41,150 +42,124 @@ from xl import (
     common,
     event,
     metadata,
-    player,
+    providers,
     settings,
     xdg
 )
-from xl.covers import MANAGER as cover_manager
+from xl.covers import MANAGER as COVER_MANAGER
 from xl.nls import gettext as _
-import xlgui
+from xlgui.widgets import dialogs
 from xlgui import (
     guiutil,
     icons
 )
+
 logger = logging.getLogger(__name__)
 
-class CoverManager(object):
+class CoverManager(gobject.GObject):
     """
         Cover manager window
     """
+    __gsignals__ = {
+        'prefetch-started': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            ()
+        ),
+        'prefetch-progress': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_INT,)
+        ),
+        'prefetch-completed': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_INT,)
+        ),
+        'fetch-started': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_INT,)
+        ),
+        'fetch-completed': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_INT,)
+        ),
+        'fetch-progress': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_INT,)
+        ),
+        'cover-fetched': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_PYOBJECT, gtk.gdk.Pixbuf)
+        )
+    }
+
     def __init__(self, parent, collection):
         """
             Initializes the window
         """
-        self.parent = parent
-        self.collection = collection
+        gobject.GObject.__init__(self)
 
-        self.cover_nodes = {}
-        self.covers = {}
-        self.track_dict = {}
-        self._stopped = True
+        # List of identifiers of albums without covers
+        self.outstanding = []
+        # Map of album identifiers and their tracks
+        self.album_tracks = {}
 
-        self.builder = gtk.Builder()
-        self.builder.add_from_file(xdg.get_data_path('ui/covermanager.ui'))
+        self.outstanding_text = _('{outstanding} covers left to fetch')
+        self.completed_text = _('All covers fetched')
+        self.cover_size = (90, 90)
+        self.default_cover_pixbuf = icons.MANAGER.pixbuf_from_data(
+            COVER_MANAGER.get_default_cover(), self.cover_size)
 
-        self.window = self.builder.get_object('CoverManager')
+        builder = gtk.Builder()
+        builder.add_from_file(xdg.get_data_path('ui/covermanager.ui'))
+        builder.connect_signals(self)
+
+        self.window = builder.get_object('window')
         self.window.set_transient_for(parent)
 
-        self.icons = self.builder.get_object('cover_icon_view')
-        self.icons.connect('button-press-event',
-            self._on_button_press)
-        self.progress = self.builder.get_object('progress')
-        self.stop_button = self.builder.get_object('stop_button')
-        self.model = gtk.ListStore(str, gtk.gdk.Pixbuf, object)
-        self.icons.set_item_width(100)
+        self.message = dialogs.MessageBar(
+            parent=builder.get_object('content_area'),
+            buttons=gtk.BUTTONS_CLOSE
+        )
 
-        self.icons.set_text_column(0)
-        self.icons.set_pixbuf_column(1)
-
-        self.nocover = icons.MANAGER.pixbuf_from_data(
-            cover_manager.get_default_cover(), (80, 80))
-
-        self._connect_events()
-        self.window.show_all()
-        glib.idle_add(self._find_initial)
+        self.previews_box = builder.get_object('previews_box')
+        self.model = builder.get_object('covers_model')
+        # Map of album identifiers and model paths
+        self.model_path_cache = {}
         self.menu = CoverMenu(self)
+        self.menu.attach_to_widget(self.previews_box, lambda menu, widget: True)
 
-    def _on_button_press(self, button, event):
-        """
-            Called when someone clicks on the cover widget
-        """
-        if event.type == gtk.gdk._2BUTTON_PRESS:
-            self.show_cover()
-        elif event.button == 3:
-            self.menu.popup(event)
+        self.progress_bar = builder.get_object('progressbar')
+        self.progress_bar.set_text(_('Collecting albums and covers...'))
+        self.progress_bar.set_data('pulse-timeout',
+            glib.timeout_add(100, self.on_progress_pulse_timeout))
+        self.close_button = builder.get_object('close_button')
+        self.stop_button = builder.get_object('stop_button')
+        self.stop_button.set_sensitive(False)
+        self.fetch_button = builder.get_object('fetch_button')
 
-        # select the current icon
-        x, y = map(int, event.get_coords())
-        path = self.icons.get_path_at_pos(x, y)
-        if path:
-            self.icons.select_path(path)
+        self.window.show_all()
 
-    def get_selected_cover(self):
-        """
-            Returns the currently selected cover tuple
-        """
-        paths = self.icons.get_selected_items()
-        if paths:
-            path = paths[0]
-            iter = self.model.get_iter(path)
-            return self.model.get_value(iter, 2)
+        self.stopper = threading.Event()
+        thread = threading.Thread(target=self.prefetch, name='CoverPrefetch', args=(collection,))
+        thread.daemon = True
+        thread.start()
 
-    def show_cover(self, *e):
+    def prefetch(self, collection):
         """
-            Shows the currently selected cover
+            Collects all albums and sets the list of outstanding items
         """
-        item = self._get_selected_item()
-        track = self.track_dict[item][0]
-        cover_data = cover_manager.get_cover(track)
+        albums = set()
 
-        pixbuf = icons.MANAGER.pixbuf_from_data(cover_data)
+        for track in collection:
+            if self.stopper.is_set():
+                return
 
-        if pixbuf:
-            window = CoverWindow(self.parent, pixbuf,
-                track.get_tag_display('title'))
-            window.show_all()
-
-    def fetch_cover(self):
-        """
-            Fetches a cover for the current track
-        """
-        item = self._get_selected_item()
-        if item:
-            track = self.track_dict[item][0]
-            window = CoverChooser(self.window, track)
-            window.connect('cover-chosen', self.on_cover_chosen)
-
-    def on_cover_chosen(self, object, cover_data):
-        paths = self.icons.get_selected_items()
-        if not paths:
-            return None
-        row = self.model[paths[0]]
-        item = row[2]
-
-        pixbuf = icons.MANAGER.pixbuf_from_data(
-            cover_data, (80, 80))
-        self.covers[item] = pixbuf
-        row[1] = pixbuf
-
-    def _get_selected_item(self):
-        """
-            Returns the selected item
-        """
-        paths = self.icons.get_selected_items()
-        if not paths:
-            return None
-        path = paths[0]
-        iter = self.model.get_iter(path)
-        item = self.model.get_value(iter, 2)
-        return item
-
-    def remove_cover(self, *e):
-        item = self._get_selected_item()
-        paths = self.icons.get_selected_items()
-        track = self.track_dict[item][0]
-        cover_manager.remove_cover(track)
-        self.covers[item] = self.nocover
-        if paths:
-            iter = self.model.get_iter(paths[0])
-            self.model.set_value(iter, 1, self.nocover)
-
-    def _find_initial(self):
-        """
-            Locates covers and sets the icons in the windows
-        """
-        items = set()
-        for track in self.collection:
             try:
                 artist = track.get_tag_raw('artist')[0]
                 album = track.get_tag_raw('album')[0]
@@ -194,134 +169,315 @@ class CoverManager(object):
             if not album or not artist:
                 continue
 
-            item = (artist, album)
+            album = (artist, album)
 
             try:
-                self.track_dict[item].append(track)
+                self.album_tracks[album].append(track)
             except KeyError:
-                self.track_dict[item] = [track]
+                self.album_tracks[album] = [track]
 
-            items.add(item)
+            albums.add(album)
 
-        self.items = list(items)
-        self.items.sort()
+        albums = list(albums)
+        albums.sort()
 
-        self.needs = 0
-        for item in self.items:
-            cover_avail = cover_manager.get_cover(self.track_dict[item][0],
-                set_only=True)
+        outstanding = []
+        # Speed up the following loop
+        get_cover = COVER_MANAGER.get_cover
+        pixbuf_from_data = icons.MANAGER.pixbuf_from_data
+        default_cover_pixbuf = self.default_cover_pixbuf
+        cover_size = self.cover_size
 
-            if cover_avail:
-                try:
-                    image = icons.MANAGER.pixbuf_from_data(
-                        cover_avail, size=(80,80))
-                except glib.GError:
-                    image = self.nocover
-                    self.needs += 1
-            else:
-                image = self.nocover
-                self.needs += 1
+        self.emit('prefetch-started')
 
-            display = "%s - %s" % item
-
-            self.cover_nodes[item] = self.model.append([display, image, item])
-            self.covers[item] = image
-        self.icons.set_model(self.model)
-        self.progress.set_text(_('%d covers to fetch') % self.needs)
-
-    def _connect_events(self):
-        """
-            Connects the various events
-        """
-        self.builder.connect_signals({
-            'on_stop_button_clicked': self._toggle_find,
-            'on_cancel_button_clicked': self._on_destroy
-        })
-
-        self.window.connect('delete-event', self._on_destroy)
-
-    @common.threaded
-    def _find_covers(self):
-        """
-            Finds covers for albums that don't already have one
-        """
-        self.count = 0
-        self._stopped = False
-        for item in self.items:
-            if self._stopped:
-                glib.idle_add(self._do_stop)
+        for i, album in enumerate(albums):
+            if self.stopper.is_set():
                 return
-            starttime = time.time()
-            if not self.covers[item] == self.nocover:
+
+            cover_data = get_cover(self.album_tracks[album][0], set_only=True)
+            cover_pixbuf = pixbuf_from_data(cover_data) if cover_data else None
+
+            try:
+                thumbnail_pixbuf = cover_pixbuf.scale_simple(*cover_size,
+                    interp_type=gtk.gdk.INTERP_BILINEAR)
+            except AttributeError: # cover_pixbuf is None
+                cover_pixbuf = thumbnail_pixbuf = default_cover_pixbuf
+                outstanding.append(album)
+
+            label = '{0} - {1}'.format(*album)
+            iter = self.model.append((album, cover_pixbuf, thumbnail_pixbuf, label))
+            self.model_path_cache[album] = self.model.get_path(iter)
+
+            self.emit('prefetch-progress', i + 1)
+
+        self.outstanding = outstanding
+        self.emit('prefetch-completed', len(self.outstanding))
+
+    def fetch(self):
+        """
+            Collects covers for all outstanding items
+        """
+        self.emit('fetch-started', len(self.outstanding))
+
+        # Speed up the following loop
+        get_cover = COVER_MANAGER.get_cover
+        save = COVER_MANAGER.save
+        pixbuf_from_data = icons.MANAGER.pixbuf_from_data
+
+        for i, album in enumerate(self.outstanding[:]):
+            if self.stopper.is_set():
+                # Allow for "fetch-completed" signal to be emitted
+                break
+
+            cover_data = get_cover(self.album_tracks[album][0], save_cover=True)
+            cover_pixbuf = pixbuf_from_data(cover_data) if cover_data else None
+
+            self.emit('fetch-progress', i + 1)
+
+            if not cover_pixbuf:
                 continue
 
-            c = cover_manager.get_cover(self.track_dict[item][0],
-                    save_cover=True)
+            self.outstanding.remove(album)
+            self.emit('cover-fetched', album, cover_pixbuf)
 
-            if c:
-                node = self.cover_nodes[item]
-                try:
-                    image = icons.MANAGER.pixbuf_from_data(c, size=(80,80))
-                except glib.GError:
-                    c = None
-                else:
-                    glib.idle_add(self.model.set_value, node, 1, image)
+            if i % 50 == 0:
+                logger.debug('Saving cover database')
+                save()
 
-            glib.idle_add(self.progress.set_fraction, float(self.count) /
-                float(self.needs))
-            glib.idle_add(self.progress.set_text, "%s/%s fetched" %
-                    (self.count, self.needs))
+        logger.debug('Saving cover database')
+        save()
 
-            self.count += 1
+        self.emit('fetch-completed', len(self.outstanding))
 
-            if self.count % 20 == 0:
-                logger.debug("Saving cover database")
-                cover_manager.save()
-
-        glib.idle_add(self._do_stop)
-
-    def _calculate_needed(self):
+    def show_cover(self):
         """
-            Calculates the number of needed covers
+            Shows the currently selected cover
         """
-        self.needs = 0
-        for item in self.items:
-            cvr = self.covers[item]
-            if cvr == self.nocover:
-                self.needs += 1
+        paths = self.previews_box.get_selected_items()
 
-    def _do_stop(self):
-        """
-            Actually stop the finder thread
-        """
-        self._calculate_needed()
-        self.progress.set_text(_('%d covers to fetch') % self.needs)
-        self.progress.set_fraction(0)
-        self._stopped = True
-        cover_manager.save()
-        self.stop_button.set_use_stock(False)
-        self.stop_button.set_label(_('Start'))
-        self.stop_button.set_image(gtk.image_new_from_stock(gtk.STOCK_YES,
-            gtk.ICON_SIZE_BUTTON))
+        if paths:
+            path = paths[0]
+            album = self.model[path][0]
 
-    def _on_destroy(self, *e):
-        self._do_stop()
-        self.window.hide()
+            cover_pixbuf = self.model[path][1]
+            title = '{0} - {1}'.format(*album)
 
-    def _toggle_find(self, *e):
+            cover_window = CoverWindow(self.window, cover_pixbuf, title)
+            cover_window.show_all()
+
+    def fetch_cover(self):
         """
-            Toggles cover finding
+            Shows the cover chooser for the currently selected album
         """
-        if self._stopped:
-            self.stop_button.set_use_stock(True)
-            self.stop_button.set_label(gtk.STOCK_STOP)
-            self._find_covers()
+        paths = self.previews_box.get_selected_items()
+
+        if paths:
+            path = paths[0]
+            album = self.model[path][0]
+            track = self.album_tracks[album][0]
+            cover_chooser = CoverChooser(self.window, track)
+            # Make sure we're updating the correct album after selection
+            cover_chooser.set_data('path', path)
+            cover_chooser.connect('cover-chosen', self.on_cover_chosen)
+
+    def remove_cover(self):
+        """
+            Removes the cover of the currently selected album
+        """
+        paths = self.previews_box.get_selected_items()
+
+        if paths:
+            path = paths[0]
+            album = self.model[path][0]
+            track = self.album_tracks[album][0]
+            COVER_MANAGER.remove_cover(track)
+            self.model[path][1] = self.model[path][2] = self.default_cover_pixbuf
+
+    def do_prefetch_started(self):
+        """
+            Sets the widget states to prefetching
+        """
+        self.previews_box.set_model(None)
+        self.previews_box.set_sensitive(False)
+        self.fetch_button.set_sensitive(False)
+        self.progress_bar.set_fraction(0)
+        glib.source_remove(self.progress_bar.get_data('pulse-timeout'))
+
+    def do_prefetch_completed(self, outstanding):
+        """
+            Sets the widget states to ready for fetching
+        """
+        self.previews_box.set_sensitive(True)
+        self.previews_box.set_model(self.model)
+        self.fetch_button.set_sensitive(True)
+        self.progress_bar.set_fraction(0)
+        self.progress_bar.set_text(self.outstanding_text.format(
+            outstanding=outstanding))
+
+    def do_prefetch_progress(self, progress):
+        """
+            Updates the wiedgets to reflect the processed album
+        """
+        fraction = progress / float(len(self.album_tracks))
+        self.progress_bar.set_fraction(fraction)
+
+    def do_fetch_started(self, outstanding):
+        """
+            Sets the widget states to fetching
+        """
+        self.previews_box.set_sensitive(False)
+        self.stop_button.set_sensitive(True)
+        self.fetch_button.set_sensitive(False)
+        self.progress_bar.set_fraction(0)
+        # We need float for the fraction during progress
+        self.progress_bar.set_data('outstanding-total', float(outstanding))
+
+    def do_fetch_completed(self, outstanding):
+        """
+            Sets the widget states to ready for fetching
+        """
+        self.previews_box.set_sensitive(True)
+        self.stop_button.set_sensitive(False)
+
+        if outstanding > 0:
+            # If there are covers left for some reason, allow re-fetch
+            self.fetch_button.set_sensitive(True)
+
+        self.progress_bar.set_fraction(0)
+
+    def do_fetch_progress(self, progress):
+        """
+            Updates the widgets to reflect the processed album
+        """
+        outstanding = len(self.outstanding)
+
+        if outstanding > 0:
+            progress_text = self.outstanding_text.format(
+                outstanding=outstanding)
         else:
-            self._stopped = True
-            self.stop_button.set_use_stock(False)
-            self.stop_button.set_label(_('Start'))
-            self.stop_button.set_image(gtk.image_new_from_stock(gtk.STOCK_YES,
-                gtk.ICON_SIZE_BUTTON))
+            progress_text = self.completed_text
+
+        self.progress_bar.set_text(progress_text)
+
+        fraction = progress / self.progress_bar.get_data('outstanding-total')
+        self.progress_bar.set_fraction(fraction)
+
+    def do_cover_fetched(self, album, pixbuf):
+        """
+            Updates the widgets to reflect the newly fetched cover
+        """
+        path = self.model_path_cache[album]
+        self.model[path][1] = pixbuf
+        self.model[path][2] = pixbuf.scale_simple(*self.cover_size,
+            interp_type=gtk.gdk.INTERP_BILINEAR)
+
+    def on_cover_chosen(self, cover_chooser, cover_data):
+        """
+            Updates the cover of the current album after user selection
+        """
+        path = cover_chooser.get_data('path')
+
+        if path:
+            album = self.model[path][0]
+            pixbuf = icons.MANAGER.pixbuf_from_data(cover_data)
+
+            self.emit('cover-fetched', album, pixbuf)
+
+            try:
+                self.outstanding.remove(album)
+            except ValueError:
+                pass
+            else:
+                outstanding = len(self.outstanding)
+
+                if outstanding > 0:
+                    progress_text = self.outstanding_text.format(
+                        outstanding=outstanding)
+                else:
+                    progress_text = self.completed_text
+
+                self.progress_bar.set_text(progress_text)
+
+    def on_previews_box_item_activated(self, iconview, path):
+        """
+            Shows the currently selected cover
+        """
+        self.show_cover()
+
+    def on_previews_box_button_press_event(self, widget, e):
+        """
+            Shows the cover menu upon click
+        """
+        path = self.previews_box.get_path_at_pos(int(e.x), int(e.y))
+
+        if path:
+            self.previews_box.select_path(path)
+
+            if e.button == 3:
+                self.menu.popup(None, None, None, 3, e.time)
+
+    def on_previews_box_popup_menu(self, menu):
+        """
+            Shows the cover menu upon keyboard interaction
+        """
+        paths = self.previews_box.get_selected_items()
+
+        if paths:
+            self.menu.popup(None, None, None, 0, gtk.get_current_event_time())
+
+    def on_previews_box_query_tooltip(self, widget, x, y, keyboard_mode, tooltip):
+        """
+            Custom tooltip display to prevent markup errors
+            (e.g. due to album names containing "<")
+        """
+        x, y = self.previews_box.convert_widget_to_bin_window_coords(x, y)
+        path = self.previews_box.get_path_at_pos(x, y)
+
+        if path:
+            tooltip.set_text(self.model[path][3])
+            self.previews_box.set_tooltip_item(tooltip, path)
+
+            return True
+
+        return False
+
+    def on_progress_pulse_timeout(self):
+        """
+            Updates the progress during prefetching
+        """
+        self.progress_bar.pulse()
+
+        return True
+
+    def on_close_button_clicked(self, button):
+        """
+            Stops the current fetching process and closes the dialog
+        """
+        self.stopper.set()
+        self.window.destroy()
+
+    def on_stop_button_clicked(self, button):
+        """
+            Stops the current fetching process
+        """
+        self.stopper.set()
+
+    def on_fetch_button_clicked(self, button):
+        """
+            Starts the cover fetching process
+        """
+        self.stopper.clear()
+        thread = threading.Thread(target=self.fetch, name='CoverFetch')
+        thread.daemon = True
+        thread.start()
+
+    def on_window_delete_event(self, window, e):
+        """
+            Stops the current fetching process and closes the dialog
+        """
+        self.close_button.clicked()
+
+        return True
 
 class CoverMenu(guiutil.Menu):
     """
@@ -357,7 +513,7 @@ class CoverWidget(gtk.EventBox):
     __gsignals__ = {
         'cover-found': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
-    def __init__(self, image):
+    def __init__(self, image, player):
         """
             Initializes the widget
 
@@ -365,6 +521,7 @@ class CoverWidget(gtk.EventBox):
             :type image: :class:`gtk.Image`
         """
         gtk.EventBox.__init__(self)
+        self._player = player
         self.image = image
         self.cover_data = None
         self.menu = CoverMenu(self)
@@ -377,9 +534,9 @@ class CoverWidget(gtk.EventBox):
         self.image.show()
 
         event.add_callback(self.on_playback_start,
-                'playback_track_start', player.PLAYER)
+                'playback_track_start', self._player)
         event.add_callback(self.on_playback_end,
-                'playback_player_end', player.PLAYER)
+                'playback_player_end', self._player)
         event.add_callback(self.on_quit_application,
                 'quit_application')
 
@@ -395,9 +552,9 @@ class CoverWidget(gtk.EventBox):
             self.filename = None
 
         event.remove_callback(self.on_playback_start,
-                'playback_track_start', player.PLAYER)
+                'playback_track_start', self._player)
         event.remove_callback(self.on_playback_end,
-                'playback_player_end', player.PLAYER)
+                'playback_player_end', self._player)
         event.remove_callback(self.on_quit_application,
                 'quit-application')
 
@@ -412,22 +569,25 @@ class CoverWidget(gtk.EventBox):
 
         if pixbuf:
             window = CoverWindow(self.parent_window, pixbuf,
-                player.PLAYER.current.get_tag_display('title'))
+                self._player.current.get_tag_display('title'))
             window.show_all()
 
     def fetch_cover(self):
         """
             Fetches a cover for the current track
         """
-        if not player.PLAYER.current: return
-        window = CoverChooser(self.parent_window, player.PLAYER.current)
+        current_track = self._player.current
+        if not current_track: 
+            return
+            
+        window = CoverChooser(self.parent_window, current_track)
         window.connect('cover-chosen', self.on_cover_chosen)
 
     def remove_cover(self):
         """
             Removes the cover for the current track from the database
         """
-        cover_manager.remove_cover(player.PLAYER.current)
+        COVER_MANAGER.remove_cover(self._player.current)
         self.set_blank()
 
     def set_blank(self):
@@ -435,47 +595,40 @@ class CoverWidget(gtk.EventBox):
             Sets the default cover to display
         """
         pixbuf = icons.MANAGER.pixbuf_from_data(
-            cover_manager.get_default_cover())
+            COVER_MANAGER.get_default_cover())
         self.image.set_from_pixbuf(pixbuf)
-        self.set_drag_enabled(False)
+        self.set_drag_source_enabled(False)
         self.cover_data = None
 
         self.emit('cover-found', None)
 
-    def set_drag_enabled(self, drag_enabled):
+    def set_drag_source_enabled(self, enabled):
         """
             Changes the behavior for drag and drop
 
-            :param drag_enabled: Whether to accept dropped data
-                and allow to drag to other applications
-            :type drag_enabled: bool
+            :param drag_enabled: Whether to  allow
+                drag to other applications
+            :type enabled: bool
         """
-        if drag_enabled == self.get_data('drag_enabled'):
+        if enabled == self.get_data('drag_source_enabled'):
             return
 
-        if drag_enabled:
-            self.drag_dest_set(gtk.DEST_DEFAULT_ALL,
-                [('text/uri-list', 0, 0)],
-                gtk.gdk.ACTION_COPY |
-                gtk.gdk.ACTION_DEFAULT |
-                gtk.gdk.ACTION_MOVE
-            )
+        if enabled:
             self.drag_source_set(gtk.gdk.BUTTON1_MASK,
                 [('text/uri-list', 0, 0)],
                 gtk.gdk.ACTION_DEFAULT |
                 gtk.gdk.ACTION_MOVE
             )
         else:
-            self.drag_dest_unset()
             self.drag_source_unset()
 
-        self.set_data('drag_enabled', drag_enabled)
+        self.set_data('drag_source_enabled', enabled)
 
     def do_button_press_event(self, event):
         """
             Called when someone clicks on the cover widget
         """
-        if player.PLAYER.current is None or self.parent_window is None:
+        if self._player.current is None or self.parent_window is None:
             return
 
         if event.type == gtk.gdk._2BUTTON_PRESS:
@@ -488,7 +641,7 @@ class CoverWidget(gtk.EventBox):
             Paints alpha transparency
         """
         opacity = 1 - settings.get_option('gui/transparency', 0.3)
-        context = self.window.cairo_create()
+        context = self.props.window.cairo_create()
         background = self.style.bg[gtk.STATE_NORMAL]
         context.set_source_rgba(
             float(background.red) / 256**2,
@@ -530,7 +683,7 @@ class CoverWidget(gtk.EventBox):
         """
             Sets the cover based on the dragged data
         """
-        if player.PLAYER.current is not None:
+        if self._player.current is not None:
             uri = selection.get_uris()[0]
             db_string = 'localfile:%s' % uri
 
@@ -546,7 +699,7 @@ class CoverWidget(gtk.EventBox):
 
             if pixbuf is not None:
                 self.image.set_from_pixbuf(pixbuf)
-                cover_manager.set_cover(player.PLAYER.current, db_string,
+                COVER_MANAGER.set_cover(self._player.current, db_string,
                     self.cover_data)
 
     def on_cover_chosen(self, object, cover_data):
@@ -557,7 +710,7 @@ class CoverWidget(gtk.EventBox):
         width = settings.get_option('gui/cover_width', 100)
         pixbuf = icons.MANAGER.pixbuf_from_data(cover_data, (width, width))
         self.image.set_from_pixbuf(pixbuf)
-        self.set_drag_enabled(True)
+        self.set_drag_source_enabled(True)
         self.cover_data = cover_data
 
         self.emit('cover-found', pixbuf)
@@ -569,8 +722,16 @@ class CoverWidget(gtk.EventBox):
             them
         """
         glib.idle_add(self.set_blank)
+        glib.idle_add(self.drag_dest_set,
+            gtk.DEST_DEFAULT_ALL,
+            [('text/uri-list', 0, 0)],
+            gtk.gdk.ACTION_COPY |
+            gtk.gdk.ACTION_DEFAULT |
+            gtk.gdk.ACTION_MOVE
+        )
+
         fetch = not settings.get_option('covers/automatic_fetching', True)
-        cover_data = cover_manager.get_cover(track, set_only=fetch)
+        cover_data = COVER_MANAGER.get_cover(track, set_only=fetch)
 
         if not cover_data:
             return
@@ -582,7 +743,20 @@ class CoverWidget(gtk.EventBox):
         """
             Called when playback stops.  Resets to the nocover image
         """
-        self.set_blank()
+        glib.idle_add(self.drag_dest_unset)
+        glib.idle_add(self.set_blank)
+
+    def on_track_tags_changed(self, e, track, tag):
+        """
+            Updates the displayed cover upon tag changes
+        """
+        if self._player.current == track:
+            cover_data = COVER_MANAGER.get_cover(track)
+
+            if not cover_data:
+                return
+
+            glib.idle_add(self.on_cover_chosen, None, cover_data)
 
     def on_quit_application(self, type, exaile, nothing):
         """
@@ -758,7 +932,16 @@ class CoverChooser(gobject.GObject):
         one out of the list
     """
     __gsignals__ = {
-        'cover-chosen': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'covers-fetched': (
+            gobject.SIGNAL_RUN_LAST,
+            None,
+            (object,)
+        ),
+        'cover-chosen': (
+            gobject.SIGNAL_RUN_LAST,
+            None,
+            (object,)
+        )
     }
     def __init__(self, parent, track, search=None):
         """
@@ -771,114 +954,172 @@ class CoverChooser(gobject.GObject):
         self.builder.connect_signals(self)
         self.window = self.builder.get_object('CoverChooser')
 
-        tempartist = track.get_tag_display('artist')
-        tempalbum = track.get_tag_display('album')
         self.window.set_title(_("Cover options for %(artist)s - %(album)s") % {
-            'artist': tempartist,
-            'album': tempalbum
+            'artist': track.get_tag_display('artist'),
+            'album': track.get_tag_display('album')
         })
         self.window.set_transient_for(parent)
 
+        self.message = dialogs.MessageBar(
+            parent=self.builder.get_object('main_container'),
+            buttons=gtk.BUTTONS_CLOSE
+        )
+        self.message.connect('response', self.on_message_response)
+
         self.track = track
         self.covers = []
         self.current = 0
 
-        self.previous_button = self.builder.get_object('previous_button')
-        self.previous_button.set_sensitive(False)
-        self.next_button = self.builder.get_object('next_button')
-        self.next_button.set_sensitive(False)
-        self.ok_button = self.builder.get_object('ok_button')
-        self.ok_button.set_sensitive(False)
-        self.box = self.builder.get_object('cover_image_box')
         self.cover = guiutil.ScalableImageWidget()
         self.cover.set_image_size(350, 350)
-        self.box.pack_start(self.cover, True, True)
 
-        self.last_search = "%s - %s"  % (tempartist, tempalbum)
+        self.cover_image_box = self.builder.get_object('cover_image_box')
 
-        self.fetch_cover(track)
+        self.loading_indicator = gtk.Alignment()
+        self.loading_indicator.props.xalign = 0.5
+        self.loading_indicator.props.yalign = 0.5
+        self.loading_indicator.set_size_request(350, 350)
+        self.cover_image_box.pack_start(self.loading_indicator)
 
-    @common.threaded
-    def fetch_cover(self, search):
+        try:
+            spinner = gtk.Spinner()
+            spinner.set_size_request(100, 100)
+            spinner.start()
+            self.loading_indicator.add(spinner)
+        except AttributeError: # Older than GTK 2.20 and PyGTK 2.22
+            self.loading_indicator.add(gtk.Label(_('Loading...')))
+
+        self.size_label = self.builder.get_object('size_label')
+        self.source_label = self.builder.get_object('source_label')
+
+        self.covers_model = self.builder.get_object('covers_model')
+        self.previews_box = self.builder.get_object('previews_box')
+        self.previews_box.set_no_show_all(True)
+        self.previews_box.hide()
+        self.previews_box.set_model(None)
+
+        self.set_button = self.builder.get_object('set_button')
+        self.set_button.set_sensitive(False)
+
+        self.window.show_all()
+
+        self.stopper = threading.Event()
+        self.fetcher_thread = threading.Thread(target=self.fetch_cover, name='Coverfetcher')
+        self.fetcher_thread.start()
+
+    def fetch_cover(self):
         """
-            Searches for a cover
+            Searches for covers for the current track
         """
-        self.covers = []
-        self.current = 0
+        db_strings = COVER_MANAGER.find_covers(self.track)
 
-        covers = cover_manager.find_covers(self.track)
-        covers = [(x, cover_manager.get_cover_data(x)) for x in covers]
+        if db_strings:
+            for db_string in db_strings:
+                if self.stopper.is_set():
+                    return
 
-        if covers:
-            self.covers = covers
+                coverdata = COVER_MANAGER.get_cover_data(db_string)
+                # Pre-render everything for faster display later
+                pixbuf = icons.MANAGER.pixbuf_from_data(coverdata)
 
-            if len(covers) > 0:
-                self.ok_button.set_sensitive(True)
-            if len(covers) > 1:
-                self.next_button.set_sensitive(True)
+                if pixbuf:
+                    self.covers_model.append([
+                        (db_string, coverdata),
+                        pixbuf,
+                        pixbuf.scale_simple(50, 50, gtk.gdk.INTERP_BILINEAR)
+                    ])
 
-            glib.idle_add(self.show_cover, covers[0])
+        self.emit('covers-fetched', db_strings)
+
+    def do_covers_fetched(self, db_strings):
+        """
+            Finishes the dialog setup after all covers have been fetched
+        """
+        if self.stopper.is_set():
+            return
+
+        self.cover_image_box.remove(self.loading_indicator)
+        self.previews_box.set_model(self.covers_model)
+
+        if db_strings:
+            self.cover_image_box.pack_start(self.cover, True, True)
+            self.cover.show()
+            self.set_button.set_sensitive(True)
+
+            # Show thumbnail bar if more than one cover was found
+            if len(db_strings) > 1:
+                self.previews_box.set_no_show_all(False)
+                self.previews_box.show_all()
+
+            # Try to select the current cover of the track, fallback to first
+            track_db_string = COVER_MANAGER.get_db_string(self.track)
+            position = db_strings.index(track_db_string) if track_db_string in db_strings else 0
+            self.previews_box.select_path((position,))
         else:
-            glib.idle_add(self.__show_no_cover_found)
-
-    def __show_no_cover_found(self):
-        message = xlgui.main.mainwindow().message
-        message.show_info(_('No covers found.'))
-
-    def on_previous_button_clicked(self, button):
-        """
-            Shows the previous cover
-        """
-        if self.current - 1 < 0:
-            return
-
-        self.current = self.current - 1
-        self.show_cover(self.covers[self.current])
-
-        if self.current + 1 < len(self.covers):
-            self.next_button.set_sensitive(True)
-
-        if self.current - 1 < 0:
-            self.previous_button.set_sensitive(False)
-
-    def on_next_button_clicked(self, button):
-        """
-            Shows the next cover
-        """
-        if self.current + 1 >= len(self.covers):
-            return
-
-        self.current = self.current + 1
-        self.show_cover(self.covers[self.current])
-
-        if self.current + 1 >= len(self.covers):
-            self.next_button.set_sensitive(False)
-
-        if self.current - 1 >= 0:
-            self.previous_button.set_sensitive(True)
+            self.builder.get_object('info_box').hide()
+            self.builder.get_object('actions_box').hide()
+            self.message.show_warning(
+                _('No covers found.'),
+                _('None of the enabled sources has a cover for this track, try enabling more sources.')
+            )
 
     def on_cancel_button_clicked(self, button):
         """
             Closes the cover chooser
         """
+        # Notify the fetcher thread to stop
+        self.stopper.set()
+
         self.window.destroy()
 
-    def on_ok_button_clicked(self, button):
+    def on_set_button_clicked(self, button):
         """
             Chooses the current cover and saves it to the database
         """
-        track = self.track
-        coverdata = self.covers[self.current]
+        paths = self.previews_box.get_selected_items()
 
-        cover_manager.set_cover(track, coverdata[0], coverdata[1])
+        if paths:
+            path = paths[0]
+            coverdata = self.covers_model[path][0]
 
-        self.emit('cover-chosen', coverdata[1])
-        self.window.destroy()
+            COVER_MANAGER.set_cover(self.track, coverdata[0], coverdata[1])
 
-    def show_cover(self, coverdata):
+            self.emit('cover-chosen', coverdata[1])
+            self.window.destroy()
+
+    def on_previews_box_selection_changed(self, iconview):
         """
-            Shows the current cover
+            Switches the currently displayed cover
         """
-        self.cover.set_image_data(coverdata[1])
-        self.window.show_all()
+        paths = self.previews_box.get_selected_items()
+
+        if paths:
+            path = paths[0]
+            db_string = self.covers_model[path][0]
+            source = db_string[0].split(':', 1)[0]
+            provider = providers.get_provider('covers', source)
+            pixbuf = self.covers_model[path][1]
+
+            self.cover.set_image_pixbuf(pixbuf)
+            self.size_label.set_text(_('{width}x{height} pixels').format(
+                width=pixbuf.get_width(), height=pixbuf.get_height()))
+            # Display readable title of the provider, fallback to its name
+            self.source_label.set_text(getattr(provider, 'title', source))
+
+            self.set_button.set_sensitive(True)
+        else:
+            self.set_button.set_sensitive(False)
+
+    def on_previews_box_item_activated(self, iconview, path):
+        """
+            Triggers selecting the current cover
+        """
+        self.set_button.clicked()
+
+    def on_message_response(self, widget, response):
+        """
+            Handles the response for closing
+        """
+        if response == gtk.RESPONSE_CLOSE:
+            self.window.destroy()
 
