@@ -24,19 +24,40 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+    Provides methods for convenient icons and image handling
+"""
+
 import cairo
 import glob
 import glib
 import gtk
+import logging
 import os
-import pango
 
-from xl import event, settings, xdg
+from xl import (
+    common,
+    event,
+    settings,
+    xdg
+)
+
+logger = logging.getLogger(__name__)
 
 class ExtendedPixbuf(gtk.gdk.Pixbuf):
     """
-        A Pixbuf wrapper class allowing for
+        A :class:`gtk.gdk.Pixbuf` wrapper class allowing for
         interaction using standard operators
+
+        Thus you can do the following:
+
+        * ``pixbuf1 + pixbuf2`` (horizontally appends ``pixbuf2`` to ``pixbuf1``)
+        * ``pixbuf * 5`` (multiplies the content of ``pixbuf``)
+        * ``pixbuf1 & pixbuf2`` (simple composition of ``pixbuf2`` on ``pixbuf1``, the desired alpha value has to be included in the pixbufs themselves)
+        * ``pixbuf1 < pixbuf2``, ``pixbuf1 > pixbuf2`` (compares the pixbuf dimensions)
+        * ``pixbuf1 == pixbuf2`` (compares the pixel data, use the *is* operator to check for identity)
+
+        Even more is possible with the provided verbose methods
     """
     def __init__(self, pixbuf):
         gtk.gdk.Pixbuf.__init__(self,
@@ -414,7 +435,7 @@ class ExtendedPixbuf(gtk.gdk.Pixbuf):
 
 def extended_pixbuf_new_from_file(filename):
     """
-        Returns a new ExtendedPixbuf containing
+        Returns a new :class:`ExtendedPixbuf` containing
         an image loaded from the specified file
 
         :param filename: the name of the file
@@ -443,13 +464,12 @@ class IconManager(object):
         self._cache = {}
 
         self.rating_active_pixbuf = extended_pixbuf_new_from_file(
-                xdg.get_data_path('images', 'star.png'))
+            xdg.get_data_path('images', 'star.png'))
         self.rating_inactive_pixbuf = extended_pixbuf_new_from_file(
             xdg.get_data_path('images', 'emptystar.png'))
-        self.rating_pixbufs = []
-        self._generate_rating_pixbufs()
 
-        event.add_callback(self.on_option_set, 'rating_option_set')
+        # nobody actually sets the rating option, so don't handle it for now
+        #event.add_callback(self.on_option_set, 'rating_option_set')
 
     def add_icon_name_from_directory(self, icon_name, directory):
         """
@@ -497,8 +517,14 @@ class IconManager(object):
         try:
             pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
             self.add_icon_name_from_pixbuf(icon_name, pixbuf, size)
-        except Exception:
+        except Exception as e:
             # Happens if, e.g., librsvg is not installed.
+            logger.warning('Failed to add icon name "{icon_name}" '
+                           'from file "{filename}": {error}'.format(
+                icon_name=icon_name,
+                filename=filename,
+                error=e.message
+            ))
             pass
 
     def add_icon_name_from_pixbuf(self, icon_name, pixbuf, size=None):
@@ -623,16 +649,25 @@ class IconManager(object):
 
             :param stock_id: an icon name
             :type stock_id: string
-            :param size: the size of the icon
-            :type size: GtkIconSize
+            :param size: the size of the icon, will be
+                tried to converted to a GTK icon size
+            :type size: int or GtkIconSize
 
             :returns: the generated pixbuf
             :rtype: :class:`gtk.gdk.Pixbuf` or None
         """
+        if type(size) == gtk.IconSize:
+            icon_size = gtk.icon_size_lookup(size)
+            size = icon_size[0]
+
         try:
             pixbuf = self.icon_theme.load_icon(
                 icon_name, size, gtk.ICON_LOOKUP_NO_SVG)
-        except glib.GError:
+        except glib.GError as e:
+            logger.warning('Failed to get pixbuf from "{icon_name}": {error}'.format(
+                icon_name=icon_name,
+                error=e.message
+            ))
             pixbuf = None
 
         # TODO: Check if fallbacks are necessary
@@ -660,11 +695,14 @@ class IconManager(object):
         if not data:
             return None
 
-        def on_size_prepared(loader, width, height):
-            """
-                Keeps the ratio if requested
-            """
-            if size is not None:
+        pixbuf = None
+        loader = gtk.gdk.PixbufLoader()
+
+        if size is not None:
+            def on_size_prepared(loader, width, height):
+                """
+                    Keeps the ratio if requested
+                """
                 if keep_ratio:
                     scale = min(size[0] / float(width), size[1] / float(height))
 
@@ -680,17 +718,20 @@ class IconManager(object):
                     else:
                         width = height = max(width, height)
 
-            loader.set_size(width, height)
+                loader.set_size(width, height)
+            loader.connect('size-prepared', on_size_prepared)
 
-        loader = gtk.gdk.PixbufLoader()
-        loader.connect('size-prepared', on_size_prepared)
         try:
             loader.write(data)
             loader.close()
-        except glib.GError:
-            return None
+        except glib.GError as e:
+            logger.warning('Failed to get pixbuf from data: {error}'.format(
+                error=e.message
+            ))
+        else:
+            pixbuf = loader.get_pixbuf()
 
-        return loader.get_pixbuf()
+        return pixbuf
 
     def pixbuf_from_text(self, text, size, background_color='#456eac',
             border_color='#000', text_color='#fff'):
@@ -747,7 +788,8 @@ class IconManager(object):
 
         return pixbuf
 
-    def pixbuf_from_rating(self, rating):
+    @common.cached(limit=settings.get_option('rating/maximum', 5)*3)
+    def pixbuf_from_rating(self, rating, size_ratio=1):
         """
             Returns a pixbuf representing a rating
 
@@ -757,35 +799,27 @@ class IconManager(object):
             :returns: the rating pixbuf
             :rtype: :class:`gtk.gdk.Pixbuf`
         """
-        rating = max(0, rating)
-        rating = min(rating, len(self.rating_pixbufs) - 1)
-
-        return self.rating_pixbufs[rating]
-
-    def _generate_rating_pixbufs(self):
-        """
-            Generates the pixbufs for
-            the various rating stages
-        """
         maximum = settings.get_option('rating/maximum', 5)
         width = self.rating_active_pixbuf.get_width()
         height = self.rating_active_pixbuf.get_height()
-
-        self.rating_pixbufs = [self.rating_inactive_pixbuf * maximum]
-
-        for n in xrange(1, maximum):
-            active_pixbufs = self.rating_active_pixbuf * n
-            inactive_pixbufs = self.rating_inactive_pixbuf * (maximum - n)
-            self.rating_pixbufs += [active_pixbufs + inactive_pixbufs]
-
-        self.rating_pixbufs += [self.rating_active_pixbuf * maximum]
-
-    def on_option_set(self, event_type, settings, option):
-        """
-            Regenerates the rating pixbufs
-        """
-        if option == 'rating/maximum':
-            self._generate_rating_pixbufs()
+        
+        active_pixbuf = self.rating_active_pixbuf.scale_simple(int(width*size_ratio),
+                                                               int(height*size_ratio),
+                                                               gtk.gdk.INTERP_BILINEAR)
+        inactive_pixbuf = self.rating_inactive_pixbuf.scale_simple(int(width*size_ratio),
+                                                                   int(height*size_ratio),
+                                                                   gtk.gdk.INTERP_BILINEAR)
+        rating = max(0, rating)
+        rating = min(rating, maximum)
+        
+        if rating == 0:
+            return inactive_pixbuf*maximum
+        elif rating == maximum:
+            return active_pixbuf*maximum
+        
+        active_pixbufs = active_pixbuf * rating
+        inactive_pixbufs = inactive_pixbuf * (maximum - rating)
+        return active_pixbufs + inactive_pixbufs
 
 MANAGER = IconManager()
 
