@@ -54,12 +54,20 @@ class Panel(gobject.GObject):
         # file://, otherwise check in the data directories
         ui_file = self.ui_info[0]
         if not os.path.isabs(ui_file):
-            ui_file = xdg.get_data_path('ui/%s' % ui_file)
+            ui_file = xdg.get_data_path('ui', 'panel', ui_file)
 
         self.builder = gtk.Builder()
         self.builder.add_from_file(ui_file)
         self._child = None
 
+    def focus(self):
+        """
+            Makes this panel grab the keyboard focus
+            Subclasses can override this to give focus to a particular widget
+            or perform another action. 
+        """
+        self._child.grab_focus()
+        
     def get_panel(self):
         if not self._child:
             window = self.builder.get_object(self.ui_info[1])
