@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,91 +24,186 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import logging
-import traceback
-import urllib
-
+import glib
 import gobject
 import gtk
+import itertools
+import logging
 
 from xl.nls import gettext as _
-from xl import event, xdg, common, metadata, settings, trax
+from xl import (
+    common,
+    event,
+    formatter,
+    settings,
+    trax,
+    xdg,
+    providers
+)
 import xlgui
-from xlgui import panel, guiutil, menu, playlist, rating
+from xlgui import (
+    guiutil,
+    icons,
+    panel
+)
+from xlgui.widgets.common import DragTreeView
+from xlgui.widgets import (
+    info,
+    menu,
+    menuitems
+)
 
 logger = logging.getLogger(__name__)
 
-TRACK_NUM = 300
-
 # TODO: come up with a more customizable way to handle this
 SEARCH_TAGS = ("artist", "albumartist", "album", "title")
 
+def __create_collection_panel_context_menu():
+    items = []
+    def sorted_get_tracks_func(panel, context):
+        tracks = context['selected-tracks']
+        # TODO: base sort order on panel sort order
+        tracks = trax.sort_tracks(common.BASE_SORT_TAGS, tracks)
+        return tracks
+    items.append(menuitems.AppendMenuItem('append', after=[]))
+    items.append(menuitems.ReplaceCurrentMenuItem('replace', 
+        after=[items[-1].name]))
+    items.append(menuitems.EnqueueMenuItem('enqueue', after=[items[-1].name], 
+            get_tracks_func=sorted_get_tracks_func))
+    items.append(menuitems.RatingMenuItem('rating', after=[items[-1].name]))
+    items.append(menu.simple_separator('sep1', after=[items[-1].name]))
+    items.append(menuitems.OpenDirectoryMenuItem('open-drectory', 
+        after=[items[-1].name]))
+
+    def collection_delete_tracks_func(panel, context, tracks):
+        panel.collection.delete_tracks(tracks)
+    items.append(menuitems.TrashMenuItem('trash-tracks',
+        after=[items[-1].name],
+        delete_tracks_func=collection_delete_tracks_func))
+    items.append(menu.simple_separator('sep2', after=[items[-1].name]))
+    items.append(menuitems.PropertiesMenuItem('properties',
+            after=[items[-1].name], get_tracks_func=sorted_get_tracks_func))
+
+    for item in items:
+        providers.register('collection-panel-context-menu', item)
+__create_collection_panel_context_menu()
+
+class CollectionContextMenu(menu.ProviderMenu):
+    def __init__(self, panel):
+        menu.ProviderMenu.__init__(self, 'collection-panel-context-menu', panel)
+
+    def get_context(self):
+        context = common.LazyDict(self._parent)
+        context['selected-tracks'] = lambda name, parent: \
+            parent.tree.get_selected_tracks()
+        return context
+
 def first_meaningful_char(s):
-    for i in range(len(s)):
-        if s[i].isdigit():
+    for c in unicode(s):
+        if c.isdigit():
             return '0'
-        elif s[i].isalpha():
-            return s[i]
-    return '_'
+        elif c.isalpha():
+            return c
+    else:
+        return '_'
 
-class TreeLevelTabs(object):
-    def __init__(self, level):
+class Order(object):
+    """
+        An Order represents a structure for arranging Tracks into the
+        Collection tree.
 
-        if type(level) is str:
-            self.__tags = [level]
-            self.__printList = [0]
-            self.__searchedTagsIndices = [0]
-            return
-        if type(level) is tuple:
-            self.__tags = level[0]
-            self.__printList = level[1]
-            self.__searchedTagsIndices = level[2]
-            return
+        It is based on a list of levels, which each take the form (("sort1",
+        "sort2"), "$displaytag - $displaytag", ("search1", "search2")) wherin
+        the first entry is a tuple of tags to use for sorting, the second a
+        format string for xl.formatter, and the third a tuple of tags to use
+        for searching.
 
-    def printTags(self, tagsValues):
-        return ''.join([(tagsValues[x] if type(x) is int else x) for x in self.__printList])
+        When passed in the paramters, a level can also be a single string
+        instead of a tuple, and it will be treated equivalently to (("foo",),
+        "$foo", ("foo",)) for some string "foo".
+    """
+    def __init__(self, name, levels, use_compilations=True):
+        self.__name = name
+        self.__levels = map(self.__parse_level, levels)
+        self.__formatters = [formatter.TrackFormatter(l[1]) for l 
+            in self.__levels]
+        self.__use_compilations = use_compilations
 
-    def tags(self):
-        return self.__tags
+    @staticmethod
+    def __parse_level(val):
+        if type(val) in (str, unicode):
+            val = ((val,), "$%s"%val, (val,))
+        return tuple(val)
 
-    def searchedTagsIndices(self):
-        return self.__searchedTagsIndices
+    @property
+    def name(self):
+        return self.__name
 
-def get_all_tags(order):
-    result = []
-    for level in order:
-        result.extend(level.tags())
-    return result
+    @property
+    def use_compilations(self):
+        return self.__use_compilations
+
+    def get_levels(self):
+        return self.__levels[:]
+
+    def __len__(self):
+        return len(self.__levels)
+
+    def __eq__(self, other):
+        self.__levels == other.get_levels()
+
+    def all_sort_tags(self):
+        return list(itertools.chain(*[l[0] for l in self.__levels]))
+
+    def get_sort_tags(self, level):
+        return list(self.__levels[level][0])
+
+    def all_search_tags(self):
+        return list(itertools.chain(*[l[2] for l in self.__levels]))
+
+    def get_search_tags(self, level):
+        return list(set(self.__levels[level][2]))
+
+    def format_track(self, level, track):
+        return self.__formatters[level].format(track)
+
+DEFAULT_ORDERS = [
+    (_("Artist"), 
+        ("artist", "album", 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+    (_("Album"), 
+        ("album", 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+    (_("Genre - Artist"), 
+        ('genre', 'artist', 'album', 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+    (_("Genre - Album"), 
+        ('genre', 'album', 'artist', 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+    (_("Date - Artist"), 
+        ('date', 'artist', 'album', 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+    (_("Date - Album"), 
+        ('date', 'album', 'artist', 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+    (_("Artist - (Date - Album)"), 
+        ('artist', 
+            (('date', 'album'), "$date - $album", ('date', 'album')), 
+            (("discnumber", "tracknumber", "title"), "$title", ("title",)))),
+        ]
 
 class CollectionPanel(panel.Panel):
     """
         The collection panel
     """
     __gsignals__ = {
-        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object, bool)),
+        'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'collection-tree-loaded': (gobject.SIGNAL_RUN_LAST, None, ()),
     }
 
-    ui_info = ('collection_panel.ui', 'CollectionPanelWindow')
-    """
-        Each level in order is a tuple of tree lists
-        First list is list of tags, that are relevant to this level, in their sort order
-        Second list describes way of node is printed. Strings in list remain the same,
-        and values of corresponding tags are inserted instead of integers
-        Third list is list of indices of tags that tracks are searched by
-        If level is string 'tag', it's the same as (['tag'], [0], [0])
-    """
-    orders = (
-        ['artist', 'album', (['discnumber', 'tracknumber', 'title'], [2], [2])],
-        ['album', (['discnumber', 'tracknumber', 'title'], [2], [2])],
-        ['genre', 'artist', 'album', (['discnumber', 'tracknumber', 'title'], [2], [2])],
-        ['genre', 'album', 'artist', (['discnumber', 'tracknumber', 'title'], [2], [2])],
-        ['date', 'artist', 'album', (['discnumber', 'tracknumber', 'title'], [2], [2])],
-        ['date', 'album', 'artist', (['discnumber', 'tracknumber', 'title'], [2], [2])],
-        ['artist', (['date', 'album'], [0, ' - ', 1], [0, 1]), (['discnumber', 'tracknumber', 'title'], [2], [2])],
-    )
-
+    ui_info = ('collection.ui', 'CollectionPanelWindow')
     def __init__(self, parent, collection, name=None,
         _show_collection_empty_message=False):
         """
@@ -131,6 +226,7 @@ class CollectionPanel(panel.Panel):
         self._refresh_id = 0
         self.start_count = 0
         self.keyword = ''
+        self.orders = map(lambda x: Order(x[0], x[1]), DEFAULT_ORDERS)
         self._setup_tree()
         self._setup_widgets()
         self._check_collection_empty()
@@ -143,87 +239,49 @@ class CollectionPanel(panel.Panel):
         event.add_callback(self._check_collection_empty, 'libraries_modified',
             collection)
 
-        self.menu = menu.CollectionPanelMenu(self.tree.get_selection(),
-            self.get_selected_tracks,
-            self.get_tracks_rating)
-        self.menu.connect('append-items', lambda *e:
-            self.emit('append-items', self.get_selected_tracks()))
-        self.menu.connect('queue-items', lambda *e:
-            self.emit('queue-items', self.get_selected_tracks()))
-        self.menu.connect('rating-set', self._on_set_rating)
-        self.menu.connect('delete-items', self._on_delete_items)
-        self.menu.connect('properties', lambda *e:
-            self.properties_dialog())
-
+        self.menu = CollectionContextMenu(self)
 
         self.load_tree()
 
-    def properties_dialog(self):
-        """
-            Shows the properties dialog
-        """
-        from xlgui import properties
-        tracks = self.get_selected_tracks()
-
-        if not tracks:
-            return False
-
-        tracks = trax.sort_tracks(
-			('artist', 'date', 'album', 'discnumber', 'tracknumber'),
-			tracks)
-
-        dialog = properties.TrackPropertiesDialog(self.parent,
-            tracks)
-
-    def _on_set_rating(self, widget, new_rating):
-        """
-            Called when a new rating is chosen from the rating menu
-        """
-        tracks = self.get_selected_tracks()
-        rating.set_rating(tracks, new_rating)
-
-    def _on_delete_items(self, *args):
-        dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,
-                buttons=gtk.BUTTONS_YES_NO,
-                message_format=_("This will permanantly delete the selected "
-                    "tracks from your disk, are you sure you wish to continue?")
-                )
-        res = dialog.run()
-        if res == gtk.RESPONSE_YES:
-            tracks = self.get_selected_tracks()
-            self.collection.delete_tracks(tracks)
-
-        dialog.destroy()
-        gobject.idle_add(self.load_tree)
-
-
     def _setup_widgets(self):
         """
             Sets up the various widgets to be used in this panel
         """
         self.choice = self.builder.get_object('collection_combo_box')
-        active = settings.get_option('gui/collection_active_view', 0)
-        self.choice.set_active(active)
+        self.choicemodel = self.builder.get_object('collection_combo_model')
+        self.repopulate_choices()
 
         self.filter = guiutil.SearchEntry(
             self.builder.get_object('collection_search_entry'))
 
+    def repopulate_choices(self):
+        self.choice.set_model(None)
+        self.choicemodel.clear()
+        for order in self.orders:
+            self.choicemodel.append([order.name])
+        self.choice.set_model(self.choicemodel)
+        # FIXME: use something other than index here, since index
+        # doesn't deal well with dynamic lists...
+        active = settings.get_option('gui/collection_active_view', 0)
+        self.choice.set_active(active)
+
     def _check_collection_empty(self, *e):
         if not self._show_collection_empty_message or \
             (self.collection.libraries and self.collection_empty_message):
             self.collection_empty_message = False
-            self.vbox.set_child_visible(True)
-            self.message.set_child_visible(False)
-            self.vbox.show_all()
-            self.message.hide_all()
+            glib.idle_add(self.vbox.set_child_visible, True)
+            glib.idle_add(self.message.set_child_visible, False)
+            glib.idle_add(self.vbox.show_all)
+            glib.idle_add(self.message.hide_all)
 
-        elif not self.collection.libraries and not self.collection_empty_message:
+        elif not self.collection.libraries and not \
+            self.collection_empty_message:
             self.collection_empty_message = True
-            self.vbox.set_child_visible(False)
-            self.message.set_no_show_all(False)
-            self.message.set_child_visible(True)
-            self.vbox.hide_all()
-            self.message.show_all()
+            glib.idle_add(self.vbox.set_child_visible, False)
+            glib.idle_add(self.message.set_no_show_all, False)
+            glib.idle_add(self.message.set_child_visible, True)
+            glib.idle_add(self.vbox.hide_all)
+            glib.idle_add(self.message.show_all)
 
     def _connect_events(self):
         """
@@ -231,40 +289,45 @@ class CollectionPanel(panel.Panel):
         """
         self.builder.connect_signals({
             'on_collection_combo_box_changed': lambda *e: self.load_tree(),
-            'on_refresh_button_pressed': self.on_refresh_button_pressed,
-            'on_refresh_button_key_pressed': self.on_refresh_button_key_pressed,
-            'on_collection_search_entry_activate': self.on_collection_search_entry_activate,
-            'on_empty_collection_button_clicked': lambda *x: xlgui.controller().collection_manager()
+            'on_refresh_button_press_event': self.on_refresh_button_press_event,
+            'on_refresh_button_key_press_event': 
+                self.on_refresh_button_key_press_event,
+            'on_collection_search_entry_activate': 
+                self.on_collection_search_entry_activate,
+            'on_add_music_button_clicked': self.on_add_music_button_clicked
         })
         self.tree.connect('key-release-event', self.on_key_released)
         event.add_callback(self.refresh_tags_in_tree, 'track_tags_changed')
+        event.add_callback(self.refresh_tracks_in_tree, 
+            'tracks_added', self.collection)
+        event.add_callback(self.refresh_tracks_in_tree, 
+            'tracks_removed', self.collection)
 
-    def on_refresh_button_pressed(self, button, event):
+    def on_refresh_button_press_event(self, button, event):
         """
             Called on mouse activation of the refresh button
         """
         if event.button == 3:
             menu = guiutil.Menu()
             menu.append(_('Rescan Collection'),
-                xlgui.controller().on_rescan_collection,
-                'gtk-refresh')
+                xlgui.get_controller().on_rescan_collection,
+                gtk.STOCK_REFRESH)
             menu.popup(None, None, None, event.button, event.time)
             return
 
         if event.state & gtk.gdk.SHIFT_MASK:
-            xlgui.controller().on_rescan_collection(None)
+            xlgui.get_controller().on_rescan_collection(None)
         else:
             self.load_tree()
 
-    def on_refresh_button_key_pressed(self, widget, event):
+    def on_refresh_button_key_press_event(self, widget, event):
         """
             Called on key presses on the refresh button
         """
-        keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname != 'Return': return False
+        if event.keyval != gtk.keysyms.Return: return False
 
         if event.state & gtk.gdk.SHIFT_MASK:
-            xlgui.controller().on_rescan_collection(None)
+            xlgui.get_controller().on_rescan_collection(None)
         else:
             self.load_tree()
 
@@ -301,21 +364,27 @@ class CollectionPanel(panel.Panel):
         self.start_count += 1
         self.load_tree()
 
+    def on_add_music_button_clicked(self, button):
+        xlgui.get_controller().collection_manager()
+
     def _setup_images(self):
         """
             Sets up the various images that will be used in the tree
         """
-        window = gtk.Window()
-        self.artist_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path("images/artist.png"))
-        self.date_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path('images/year.png'))
-        self.album_image = window.render_icon('gtk-cdrom',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.title_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path('images/track.png'))
-        self.genre_image = gtk.gdk.pixbuf_new_from_file(xdg.get_data_path('images/genre.png'))
+        self.artist_image = icons.MANAGER.pixbuf_from_icon_name(
+            'artist', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.date_image = icons.MANAGER.pixbuf_from_icon_name(
+            'office-calendar', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.album_image = icons.MANAGER.pixbuf_from_stock(
+            gtk.STOCK_CDROM, gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.title_image = icons.MANAGER.pixbuf_from_icon_name(
+            'audio-x-generic', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.genre_image = icons.MANAGER.pixbuf_from_icon_name(
+            'genre', gtk.ICON_SIZE_SMALL_TOOLBAR)
 
     def drag_data_received(self, *e):
         """
-            stubb
+            stub
         """
         pass
 
@@ -329,17 +398,19 @@ class CollectionPanel(panel.Panel):
         """
             Called when a drag source wants data for this drag operation
         """
-        trs = self.get_selected_tracks()
-        for track in trs:
-            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
-        urls = guiutil.get_urls_for(trs)
-        selection.set_uris(urls)
+        tracks = treeview.get_selected_tracks()
+
+        for track in tracks:
+            DragTreeView.dragged_data[track.get_loc_for_io()] = track
+
+        uris = trax.util.get_uris_from_tracks(tracks)
+        selection.set_uris(uris)
 
     def _setup_tree(self):
         """
             Sets up the tree widget
         """
-        self.tree = guiutil.DragTreeView(self)
+        self.tree = CollectionDragTreeView(self)
         self.tree.set_headers_visible(False)
         container = self.builder.get_object('CollectionPanel')
         scroll = gtk.ScrolledWindow()
@@ -364,13 +435,11 @@ class CollectionPanel(panel.Panel):
             import pango
             cell.set_property('ellipsize-set', True)
             cell.set_property('ellipsize', pango.ELLIPSIZE_END)
-            self.tree.set_tooltip_column(1)
 
         self.tree.set_row_separator_func(
             lambda m, i: m.get_value(i, 1) is None)
 
         self.model = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
-        self.model_blank = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
 
         self.tree.connect("row-expanded", self.on_expanded)
 
@@ -384,75 +453,14 @@ class CollectionPanel(panel.Panel):
         srtrs = trax.search_tracks(self.tracks, [matcher])
         return [ x.track for x in srtrs ]
 
-    def get_selected_tracks(self):
-        """
-            Finds all the selected tracks
-        """
-
-        selection = self.tree.get_selection()
-        (model, paths) = selection.get_selected_rows()
-        trs = []
-        for path in paths:
-            iter = self.model.get_iter(path)
-            newset = self._find_tracks(iter)
-            trs.append(newset)
-
-        if not trs: return None
-
-        trs = list(set(reduce(lambda x, y: list(x) + list(y), trs)))
-
-        return trs
-
-    def get_tracks_rating(self):
-        """
-            Returns the rating of the selected tracks
-            Returns 0 if there is no selection, if tracks have different ratings
-            or if the selection is too big
-        """
-        rating = 0
-        selection = self.tree.get_selection()
-        (model, paths) = selection.get_selected_rows()
-        tracks_limit = settings.get_option('miscellaneous/rating_widget_tracks_limit', 0)
-        if tracks_limit == 0: return 0
-        current_count = 0
-
-        if paths and paths[0]:
-            iter = self.model.get_iter(paths[0])
-            newset = self._find_tracks(iter)
-            current_count += len (newset)
-            if current_count > tracks_limit:
-                return 0 # too many tracks
-
-            if newset and newset[0]:
-                rating = newset[0].get_rating ()
-
-            if rating == 0:
-                return 0 # if first song has 0 as a rating, we know the result
-
-            for song in newset:
-                if song.get_rating() != rating:
-                    return 0 # different ratings
-        else:
-            return 0 # no tracks
-
-        for path in paths[1:]:
-            iter = self.model.get_iter(path)
-            newset = self._find_tracks(iter)
-            current_count += len (newset)
-            if current_count > tracks_limit:
-                return 0 # too many tracks
-
-            for song in newset:
-                if song.get_rating() != rating:
-                    return 0 # different ratings
-
-        return rating # only one rating in the tracks, returning it
-
-    def append_to_playlist(self, item=None, event=None):
+    def append_to_playlist(self, item=None, event=None, replace=False):
         """
             Adds items to the current playlist
         """
-        self.emit('append-items', self.get_selected_tracks())
+        if replace:
+            self.emit('replace-items', self.tree.get_selected_tracks())
+        else:
+            self.emit('append-items', self.tree.get_selected_tracks(), True)
 
     def button_press(self, widget, event):
         """
@@ -462,9 +470,21 @@ class CollectionPanel(panel.Panel):
         (x, y) = map(int, event.get_coords())
         path = self.tree.get_path_at_pos(x, y)
         if event.type == gtk.gdk._2BUTTON_PRESS:
-            self.append_to_playlist()
+            replace = settings.get_option('playlist/replace_content', False)
+            self.append_to_playlist(replace=replace)
             return False
-        elif event.button == 3:
+        elif event.button == 2:
+            self.append_to_playlist(replace=True)
+            return False
+
+    def button_release(self, widget, event):
+        """
+            Called when the user releases the mouse button on the tree
+        """
+        selection = self.tree.get_selection()
+        (x, y) = map(int, event.get_coords())
+        path = self.tree.get_path_at_pos(x, y)
+        if event.button == 3:
             self.menu.popup(event)
             if not path:
                 return False
@@ -500,23 +520,24 @@ class CollectionPanel(panel.Panel):
         return " ".join(queries)
 
     def refresh_tags_in_tree(self, type, track, tag):
-        """
-            wrapper so that multiple events dont cause multiple
-            reloads in quick succession
-        """
         if settings.get_option('gui/sync_on_tag_change', True) and \
-            tag in get_all_tags(self.order) and \
+            tag in self.order.all_sort_tags() and \
             self.collection.loc_is_member(track.get_loc_for_io()):
-            if self._refresh_id != 0:
-                gobject.source_remove(self._refresh_id)
-            self._refresh_id = gobject.timeout_add(500,
-                    self._refresh_tags_in_tree)
+            self._refresh_tags_in_tree()
 
+    def refresh_tracks_in_tree(self, type, obj, loc):
+        glib.idle_add(self._refresh_tags_in_tree)
+
+    @common.glib_wait(500)
     def _refresh_tags_in_tree(self):
         """
             Callback for when tags have changed and the tree
             needs reloading.
         """
+        # Trying to reload while we're rescanning is really inefficient,
+        # so we delay it until we're done scanning.
+        if self.collection._scanning:
+            return True
         self.resort_tracks()
         self.load_tree()
         return False
@@ -524,7 +545,8 @@ class CollectionPanel(panel.Panel):
     def resort_tracks(self):
 #        import time
 #        print "sorting...", time.clock()
-        self.sorted_tracks = trax.sort_tracks(self.order[0].tags(), self.collection.get_tracks())
+        self.sorted_tracks = trax.sort_tracks(self.order.get_sort_tags(0),
+            self.collection.get_tracks())
 #        print "sorted.", time.clock()
 
     def load_tree(self):
@@ -534,14 +556,16 @@ class CollectionPanel(panel.Panel):
             Loads tracks based on the current keyword, or all the tracks in
             the collection associated with this panel
         """
+        logger.debug("Reloading collection tree")
         self.current_start_count = self.start_count
+        self.tree.set_model(None)
         self.model.clear()
-        self.tree.set_model(self.model_blank)
 
         self.root = None
         oldorder = self.order
-        self.order = [TreeLevelTabs(x) for x in self.orders[self.choice.get_active()]]
-        if oldorder != self.order:
+        self.order = self.orders[self.choice.get_active()]
+
+        if not oldorder or oldorder != self.order:
             self.resort_tracks()
 
         # save the active view setting
@@ -551,7 +575,8 @@ class CollectionPanel(panel.Panel):
 
         keyword = self.keyword.strip()
         tags = list(SEARCH_TAGS)
-        tags += [t for t in get_all_tags(self.order) if t != 'tracknumber' and t not in tags]
+        tags += self.order.all_search_tags()
+        tags = list(set(tags)) # uniquify list to speed up search
 
         self.tracks = list(
                 trax.search_tracks_from_string(self.sorted_tracks,
@@ -591,7 +616,7 @@ class CollectionPanel(panel.Panel):
 
         if rest:
             item = rest.pop(0)
-            gobject.idle_add(self._expand_node_by_name, search_num,
+            glib.idle_add(self._expand_node_by_name, search_num,
                 parent, item, rest)
 
     def load_subtree(self, parent):
@@ -606,7 +631,8 @@ class CollectionPanel(panel.Panel):
             depth = 0
         else:
             if self.model.iter_n_children(parent) != 1 or \
-                self.model.get_value(self.model.iter_children(parent), 1) != None:
+                self.model.get_value(
+                    self.model.iter_children(parent), 1) != None:
                 previously_loaded = True
             iter_sep = self.model.iter_children(parent)
             depth = self.model.iter_depth(parent) + 1
@@ -616,10 +642,11 @@ class CollectionPanel(panel.Panel):
         search = self.get_node_search_terms(parent)
 
         try:
-            tags = self.order[depth].tags()
+            tags = self.order.get_sort_tags(depth)
             matchers = [trax.TracksMatcher(search)]
             srtrs = trax.search_tracks(self.tracks, matchers)
-            # sort only if we are not on top level, because tracks are already sorted by fist order
+            # sort only if we are not on top level, because tracks are 
+            # already sorted by fist order
             if depth > 0:
                 srtrs = trax.sort_result_tracks(tags, srtrs)
         except IndexError:
@@ -633,11 +660,13 @@ class CollectionPanel(panel.Panel):
             bottom = True
 
 
+        display_counts = settings.get_option('gui/display_track_counts', True)
         draw_seps = settings.get_option('gui/draw_separators', True)
         last_char = ''
         last_val = ''
         last_dval = ''
         last_matchq = ''
+        count = 0
         first = True
         path = None
         expanded = False
@@ -645,19 +674,28 @@ class CollectionPanel(panel.Panel):
 
         for srtr in srtrs:
             stagvals = [unicode(srtr.track.get_tag_sort(x)) for x in tags]
-            stagval = self.order[depth].printTags(stagvals)
-            if last_val != stagval:
-                tagvals = [srtr.track.get_tag_display(x) for x in tags]
-                tagval = self.order[depth].printTags(tagvals)
+            stagval = " ".join(stagvals)
+            if (last_val != stagval or bottom):
+                tagval = self.order.format_track(depth, srtr.track)
                 match_query = " ".join([
                     srtr.track.get_tag_search(t, format=True) for t in tags])
+                if bottom:
+                    match_query += " " + \
+                            srtr.track.get_tag_search("__loc", format=True)
 
                 # Different *sort tags can cause stagval to not match
                 # but the below code will produce identical entries in
                 # the displayed tree.  This condition checks to ensure
                 # that new entries are added if and only if they will
                 # display different results, avoiding that problem.
-                if match_query != last_matchq or tagval != last_dval:
+                if match_query != last_matchq or tagval != last_dval or bottom:
+                    if display_counts and path and not bottom:
+                        iter = self.model.get_iter(path)
+                        val = self.model.get_value(iter, 1)
+                        val = "%s (%s)"%(val, count)
+                        self.model.set_value(iter, 1, val)
+                        count = 0
+
                     last_val = stagval
                     last_dval = tagval
                     if depth == 0 and draw_seps:
@@ -672,36 +710,131 @@ class CollectionPanel(panel.Panel):
                     first = False
 
                     last_matchq = match_query
-                    iter = self.model.append(parent, [image, tagval, match_query])
+                    iter = self.model.append(parent, 
+                        [image, tagval, match_query])
                     path = self.model.get_path(iter)
                     expanded = False
                     if not bottom:
                         self.model.append(iter, [None, None, None])
-
+            count += 1
             if not expanded:
                 alltags = []
-                for o in self.order[depth+1:]:
-                    alltags.extend(o.tags())
+                for i in range(depth+1, len(self.order)):
+                    alltags.extend(self.order.get_sort_tags(i))
                 for t in alltags:
                     if t in srtr.on_tags:
+                        # keep original path intact for following block
+                        newpath = path
                         if depth > 0:
                             # for some reason, nested iters are always
                             # off by one in the terminal entry.
-                            path = path[:-1] + (path[-1]-1,)
-                        to_expand.append(path)
+                            newpath = newpath[:-1] + (newpath[-1]-1,)
+                        to_expand.append(newpath)
                         expanded = True
 
+        if display_counts and path and not bottom:
+            iter = self.model.get_iter(path)
+            val = self.model.get_value(iter, 1)
+            val = "%s (%s)"%(val, count)
+            self.model.set_value(iter, 1, val)
+            count = 0
+
         if settings.get_option("gui/expand_enabled", True) and \
             len(to_expand) < \
                     settings.get_option("gui/expand_maximum_results", 100) and \
             len(self.keyword.strip()) >= \
                     settings.get_option("gui/expand_minimum_term_length", 2):
             for row in to_expand:
-                gobject.idle_add(self.tree.expand_row, row, False)
+                glib.idle_add(self.tree.expand_row, row, False)
 
         if iter_sep is not None:
             self.model.remove(iter_sep)
 
+class CollectionDragTreeView(DragTreeView):
+    """
+        Custom DragTreeView to retrieve data
+        from collection tracks
+    """
+    def __init__(self, container, receive=False, source=True):
+        """
+            :param container: The container to place the TreeView into
+            :param receive: True if the TreeView should receive drag events
+            :param source: True if the TreeView should send drag events
+        """
+        DragTreeView.__init__(self, container, receive, source)
 
+        self.set_has_tooltip(True)
+        self.connect('query-tooltip', self.on_query_tooltip)
+        # TODO: Make faster
+        #self.tooltip = CollectionToolTip(self)
+
+    def get_selected_tracks(self):
+        """
+            Returns the currently selected tracks
+        """
+        model, paths = self.get_selection().get_selected_rows()
+        tracks = []
+
+        for path in paths:
+            iter = model.get_iter(path)
+            newset = self.container._find_tracks(iter)
+            tracks.append(newset)
+
+        if not tracks: return None
+
+        tracks = list(set(reduce(lambda x, y: list(x) + list(y), tracks)))
+
+        return tracks
+
+    def on_query_tooltip(self, widget, x, y, keyboard_mode, tooltip):
+        """
+            Sets up a basic tooltip
+            Required to have "&" in tooltips working
+        """
+        if not widget.get_tooltip_context(x, y, keyboard_mode):
+            return False
+
+        path = widget.get_path_at_pos(x, y)[0]
+
+        model = widget.get_model()
+        tooltip.set_text(model[path][1]) # 1: title
+        widget.set_tooltip_row(tooltip, path)
+
+        return True
+
+class CollectionToolTip(info.TrackListToolTip):
+    """
+        Custom collection specific tooltip
+    """
+    def __init__(self, parent):
+        """
+            :param parent: the parent widget the tooltip
+                should be attached to
+        """
+        info.TrackListToolTip.__init__(self, parent, display_tracklist=True)
+
+    def on_query_tooltip(self, tree, x, y, keyboard_mode, tooltip):
+        """
+            Determines if the tooltip should be shown
+            and feeds the required data to it
+        """
+        path = tree.get_path_at_pos(x, y)
+
+        if path is None:
+            return False
+
+        path = path[0]
+        model = tree.get_model()
+        iter = model.get_iter(path)
+        tracks = tree.container._find_tracks(iter)
+
+        self.clear()
+        self.set_tracklist(tracks)
+
+        info.TrackListToolTip.on_query_tooltip(
+            self, tree, x, y, keyboard_mode, tooltip)
+        #tree.set_tooltip_row(tooltip, path)
+
+        return True
 
 # vim: et sts=4 sw=4
