@@ -43,10 +43,11 @@ import xlgui.panel.playlists as playlistpanel
 from xlgui import (
     guiutil,
     icons,
-    menu,
+    oldmenu as menu,
     panel,
     playlist as guiplaylist
 )
+from xlgui.widgets.common import DragTreeView
 from xlgui.widgets import dialogs
 
 class RadioException(Exception): pass
@@ -58,13 +59,13 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
     """
     __gsignals__ = {
         'playlist-selected': (gobject.SIGNAL_RUN_LAST, None, (object,)),
-        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object, bool)),
         'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
     __gsignals__.update(playlistpanel.BasePlaylistPanelMixin._gsignals_)
     
-    ui_info = ('radio_panel.ui', 'RadioPanelWindow')
+    ui_info = ('radio.ui', 'RadioPanelWindow')
     _radiopanel = None
 
     def __init__(self, parent, collection,
@@ -85,8 +86,8 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
 
         self._setup_tree()
         self._setup_widgets()
-        self.playlist_image = gtk.gdk.pixbuf_new_from_file(
-            xdg.get_data_path('images/playlist.png'))
+        self.playlist_image = icons.MANAGER.pixbuf_from_icon_name(
+            'music-library', gtk.ICON_SIZE_SMALL_TOOLBAR)
 
         # menus
         self.playlist_menu = menu.RadioPanelPlaylistMenu()
@@ -113,7 +114,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             self.add_driver(value)
 
     def _add_driver_cb(self, type, object, driver):
-        self.add_driver(driver)
+        glib.idle_add(self.add_driver, driver)
 
     def add_driver(self, driver):
         """
@@ -130,7 +131,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             self.tree.expand_row(self.model.get_path(node), False)
 
     def _remove_driver_cb(self, type, object, driver):
-        self.remove_driver(driver)
+        glib.idle_add(self.remove_driver, driver)
 
     def remove_driver(self, driver):
         """
@@ -164,15 +165,15 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         menu.connect('add-playlist', self._on_add_button_clicked)
 
         menu.connect('append-items', lambda *e:
-            self.emit('append-items', self.tree.get_selected_tracks()))
+            self.emit('append-items', self.tree.get_selected_tracks(), False))
         menu.connect('replace-items', lambda *e:
             self.emit('replace-items', self.tree.get_selected_tracks()))
         menu.connect('queue-items', lambda *e:
             self.emit('queue-items', self.tree.get_selected_tracks()))
         menu.connect('open-playlist', lambda *e:
             self.open_selected_playlist())
-        menu.connect('export-playlist', lambda widget, path:
-            self.export_selected_playlist(path))
+        menu.connect('export-playlist', lambda widget:
+            self.export_selected_playlist())
         menu.connect('rename-playlist', lambda widget, name:
             self.rename_selected_playlist(name))
         menu.connect('remove-playlist', lambda *e:
@@ -224,11 +225,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         else:
             pl = playlist.Playlist(name)
             tracks = trax.get_tracks_from_uri(uri)
-            try:
-                pl.add_tracks(tracks)
-            # Catch empty directories
-            except IndexError:
-                pass
+            pl.extend(tracks)
 
         self.playlist_manager.save_playlist(pl)
         self._add_to_tree(pl)
@@ -266,8 +263,8 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         self.model = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
         self.tree.set_model(self.model)
 
-        self.track = gtk.gdk.pixbuf_new_from_file(
-            xdg.get_data_path('images/track.png'))
+        self.track = icons.MANAGER.pixbuf_from_icon_name(
+            'audio-x-generic', gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.folder = self.tree.render_icon(
             gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.refresh_image = icons.MANAGER.pixbuf_from_stock(
@@ -346,7 +343,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
 
         return False
 
-    def button_press(self, widget, event):
+    def button_release(self, widget, event):
         """
             Called when someone clicks on the tree
         """
@@ -405,7 +402,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
                 self._add_new_station(locs)
                 return
             (tracks, playlists) = self.tree.get_drag_data(locs)
-            current_playlist.add_tracks(tracks)
+            current_playlist.extend(tracks)
             # Do we save in the case when a user drags a file onto a playlist in the playlist panel?
             # note that the playlist does not have to be open for this to happen
             self.playlist_manager.save_playlist(current_playlist, overwrite=True)
@@ -442,7 +439,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
                 if not name == "":
                     #Create the playlist from all of the tracks
                     new_playlist = xl.playlist.Playlist(name)
-                    new_playlist.add_tracks(tracks)
+                    new_playlist.extend(tracks)
                     self.playlist_nodes[new_playlist] = self.model.append(self.custom,
                         [self.playlist_image,
                         new_playlist.name, new_playlist])
@@ -461,7 +458,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             return
 
         for track in tracks:
-            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
+            DragTreeView.dragged_data[track.get_loc_for_io()] = track
 
         uris = trax.util.get_uris_from_tracks(tracks)
         selection_data.set_uris(uris)
