@@ -197,6 +197,32 @@ class CheckConditional(Conditional):
         """
         return self.condition_widget.get_active()
 
+
+class Button(Preference):
+    """
+        Represents a button for custom usage
+    """
+    def __init__(self, preferences, widget):
+        Preference.__init__(self, preferences, widget)
+
+        widget.connect('clicked', self.on_clicked)
+
+    def _setup_change(*e):
+        pass
+
+    def _get_value(self):
+        return None
+
+    def _set_value(self):
+        pass
+
+    def apply(*e):
+        return False
+
+    def on_clicked(self, button):
+        """ Override """
+        pass
+
 class HashedPreference(Preference):
     """
         Represents a text entry with automated hashing
@@ -313,7 +339,7 @@ class DirPreference(Preference):
         Preference.__init__(self, preferences, widget)
 
     def _setup_change(self):
-        pass
+        self.widget.connect('current-folder-changed', self.change)
 
     def _set_value(self):
         """
@@ -334,7 +360,6 @@ class OrderListPreference(Preference):
     """
     def __init__(self, preferences, widget):
         self.model = gtk.ListStore(str)
-        self.items = []
         Preference.__init__(self, preferences, widget)
         widget.set_headers_visible(False)
         widget.set_reorderable(True)
@@ -362,10 +387,10 @@ class OrderListPreference(Preference):
         """
             Value to be stored into the settings file
         """
-        self.items = []
+        items = []
 
         for row in self.model:
-            self.items += [row[0]]
+            items += [row[0]]
 
         return items
 
@@ -411,7 +436,7 @@ class SelectionListPreference(Preference):
             bool, # 3: enabled
             bool  # 4: fixed
         )
-
+        
         for item in self.items:
             self.model.append([item.id, item.title, item.description,
                 True, item.fixed])
@@ -471,6 +496,9 @@ class SelectionListPreference(Preference):
         if not available_items:
             return
 
+        # Filter out invalid items
+        selected_items = [item for item in selected_items \
+            if item in available_items]
         # Cut out unselected items
         unselected_items = [item for item in available_items \
             if item not in selected_items]
@@ -610,7 +638,7 @@ class ShortcutListPreference(Preference):
 
         try:
             cell.set_property('text', self.available_items[key])
-        except:
+        except KeyError:
             pass
 
     def on_accel_cleared(self, cellrenderer, path):
@@ -710,9 +738,9 @@ class ListPreference(Preference):
         items = self.preferences.settings.get_option(self.name,
             default=self.default)
         try:
-            items = " ".join(items)
-        except:
-            items = ""
+            items = u" ".join(items)
+        except TypeError:
+            items = u""
         self.widget.set_text(items)
 
     def _get_value(self):
@@ -770,7 +798,7 @@ class IntPreference(FloatPreference):
 
 class ColorButtonPreference(Preference):
     """
-        A class to represent the color button in the preferences window
+        A class to represent the color button
     """
     def __init__(self, preferences, widget):
         Preference.__init__(self, preferences, widget)
@@ -779,15 +807,32 @@ class ColorButtonPreference(Preference):
         self.widget.connect('color-set', self.change)
 
     def _set_value(self):
-        self.widget.set_color(gtk.gdk.color_parse(
-            self.preferences.settings.get_option(self.name,
-            self.default)))
+        value = self.preferences.settings.get_option(
+            self.name, self.default)
+
+        # Extract alpha value in any case
+        if len(value) == 9:
+            alpha = int(value[-2:], 16) * 256
+            value = value[:-2]
+
+        if self.widget.get_use_alpha():
+            self.widget.set_alpha(alpha)
+
+        self.widget.set_color(gtk.gdk.color_parse(value))
 
     def _get_value(self):
         color = self.widget.get_color()
-        string = "#%.2x%.2x%.2x" % (color.red / 257, color.green / 257,
-            color.blue / 257)
-        return string
+        value = '#%.2x%.2x%.2x' % (
+            color.red / 256,
+            color.green / 256,
+            color.blue / 256
+        )
+
+        if self.widget.get_use_alpha():
+            alpha = self.widget.get_alpha()
+            value += '%.2x' % (alpha / 256)
+
+        return value
 
 class FontButtonPreference(ColorButtonPreference):
     """
@@ -807,6 +852,23 @@ class FontButtonPreference(ColorButtonPreference):
     def _get_value(self):
         font = self.widget.get_font_name()
         return font
+        
+class FontResetButtonPreference(Button, Conditional):
+    '''
+        A button to reset a font button to a default font
+    '''
+    def __init__(self, preferences, widget):
+        Button.__init__(self, preferences, widget)
+        Conditional.__init__(self)
+
+    def on_check_condition(self):
+        if self.condition_widget.get_font_name() == self.default:
+            return False
+        return True
+
+    def on_clicked(self, button):
+        self.condition_widget.set_font_name(self.default)
+        self.condition_widget.emit('font-set')
 
 class ComboPreference(Preference):
     """
@@ -900,7 +962,7 @@ class ComboEntryPreference(Preference):
 
             completion.set_popup_single_match(True)
             completion.set_model(self.completion_list)
-            self.widget.child.set_completion(completion)
+            self.widget.get_child().set_completion(completion)
 
             for item in completion_items:
                 self.completion_list.append(item)
@@ -920,20 +982,20 @@ class ComboEntryPreference(Preference):
             Sets the preferences for this widget
         """
         value = self.preferences.settings.get_option(self.name, self.default)
-        self.widget.child.set_text(str(value))
+        self.widget.get_child().set_text(str(value))
 
     def _get_value(self):
         """
             Value to be stored into the settings file
         """
-        return self.widget.child.get_text()
+        return self.widget.get_child().get_text()
 
     def on_matching(self, completion, text, iter):
         """
             Matches the content of this box to
             the list of available completions
         """
-        cursor_pos = self.widget.child.get_position()
+        cursor_pos = self.widget.get_child().get_position()
         # Ignore the rest, allows for completions in the middle
         text = text[:cursor_pos]
         match = self.completion_list.get_value(iter, 0)
@@ -954,21 +1016,21 @@ class ComboEntryPreference(Preference):
         """
             Inserts the selected completion
         """
-        cursor_pos = self.widget.child.get_position()
-        text = self.widget.child.get_text()[:cursor_pos]
+        cursor_pos = self.widget.get_child().get_position()
+        text = self.widget.get_child().get_text()[:cursor_pos]
         match = list.get_value(iter, 0)
 
         for i in range(len(match), -1, -1):
             match_pos = text.rfind(match[:i])
             if match[:i] and match[:i] == text[match_pos:]:
                 # Delete halfway typed text
-                self.widget.child.delete_text(
+                self.widget.get_child().delete_text(
                     match_pos, match_pos + len(match[:i]))
                 # Insert match at matched position
-                self.widget.child.insert_text(
+                self.widget.get_child().insert_text(
                     match, match_pos)
                 # Update cursor position
-                self.widget.child.set_position(match_pos + len(match))
+                self.widget.get_child().set_position(match_pos + len(match))
 
         return True
 
